      Program metcomp

c----------------------------------------------------------------------
c --- METCOMP -- Meteorological Time-Series Comparision Processor
c----------------------------------------------------------------------
c
c --- METCOMP   Version: 3.21     Level: 110225                    MAIN
c
c     Copyright (c) 2009-2011 by Exponent, Inc.
c
c --- Written by:  Zhong-Xiang Wu and Joseph Scire (WDCOMP)
c                  David Strimaitis (METCOMP)
c
c --- Purpose:
c       Compute comparison statistics for winds observed at stations
c       and extracted fom modeled fields.  The statistics are from
c       METSTAT protocols and measures.
c       (Developed from WDCOMP Version 2.0, Level 060605)
c
c --- UPDATES:
c
c --- Version 3.2, Level 090729 to Version 3.21, Level 110225 (DGS)
c       1. Updated CALUTILS.FOR to version v2.58 (110225)
c          - Add control file variable type 5 (character array) and
c            retain commas in string returned so that array of
c            variable values can be parsed
c           Modified: READIN, ALTONU, SETVAR
c
c --- Version 3.12, Level 090409 to Version 3.2, Level 090729 (DGS)
c       1.  Shift times of data read from each timeseries file to
c           the base time zone assigned in the control file, if
c           the zones are different.  This allows a mix of time
c           zones to be processed (METCOMP had halted before).
c
c --- Version 3.11, Level 090320 to Version 3.12, Level 090409 (DGS)
c       1.  CALUTILS from v2.56 to v2.57 (090202)
c           Increase control file line length to 200 characters
c           Activate CPU clock using F95 system routine
c
c --- Version 3.1, Level 090129 to Version 3.11, Level 090320 (DGS)
c       1.  Add TIMESERIES.TSF 1.3 input format
c
c --- Version 3.0, Level 081210 to Version 3.1, Level 090129 (DGS)
c       1.  Update METSERIES processing to allow dataset
c           MET_TS 1.1 and TIMESERIES.TSF 1.2
c
c --- Version 2.0, Level 080110 to Version 3.0, Level 081210 (DGS)
c       1.  Introduce dataset version 1.0 structure for control file
c           and remove the 2 older control file formats.
c       2.  Introduce a date-time window for processing
c       3.  Allow file pathnames to be 320 characters long with
c           internal spaces.
c
c --- Version 1.1, Level 060630 to Version 2.0, Level 080110
c     Zhong-Xiang Wu
c     Note: Time series from WDEXTRA have NOT well considered. It 
c           may need further checks.
c
c     1. Add wind rose frequency calculations.
c     2. Add option of filtering missing data in companion 
c        time series.
c     3. Option to define input file names
c     4. Add Gross error for hourly statistics
c
c --- Version 1.0, Level 060612 to Version 1.1, Level 060630 (DGS)
c
c       1.  Add CALM threshold wind speed value that converts very low
c           wind speed occurrences (observed or modeled) to "calms".
c           The statistis for wind speed will use 0.0 m/s for such
c           cases, and the corresponding wind direction will be
c           excluded from the wind direction statistics.  Note that a
c           calm threshold of 0.0 m/s implicitly removes this screen.
c
c       2.  Tally counts of calm occurrences as total number of periods
c           observed, total number of periods modeled, and total number
c           of coincident (space/time) calm periods.
c
c       3.  Add summary tabulation of the distribution of hourly and 
c           daily statistical measures.  This is written to hourly and
c           daily output files.
c
c       4.  Add summary of "benchmark" comparisions to daily output
c           file and list file  (revise structure of list file).
c
c----------------------------------------------------------------------

      include 'metcomp.par'
      include 'metcomp.cmn'

      parameter(iecho=0)

      real xvar(mxvars)

      integer id(mxfil),itsver(2,mxfil)
      integer nvars(2,mxfil)
      integer kws(2,mxfil),kwd(2,mxfil)
      integer ktk(2,mxfil),kqq(2,mxfil)

      character*320 char320,blnk320
      character*80 title,buff,hdr1,hdr2,hdr3,blnk80
      character*80 comment1,buff1
      character*12 cver,clev
      character*64 datamod
      character*16 dataset,dataver
      character*14 cstart,cstop

      character*16 tsernam,tserver
      character*64 tsermod
      character*80 comment
      character*12 avar(mxvars)
      character*80 chrdop(3,2,mxfil)
      character*8 azone,azone8,pmap8
      character*1 cyes
      character*5 cstn
      logical lwind,ltmpk,lshum

      integer nodata
      integer iyr(24),imon(24),iday(24),ihour(24)
      integer iyrp(24),imonp(24),idayp(24),ihourp(24)
      integer kyre(24),kjule(24),khre(24),ksece(24),kdahre(24)

c --- Time shift (integer hours) ADDED to times read from file
c --- to convert to the base time zone stated in the control file
c --- A value is needed for each input data file
c ---     First index:  1=OBS, 2=PRED
      integer ihradd(2,mxfil)

      real wss(mxhours),wds(mxhours)
      real wssbkp(mxhours),wdsbkp(mxhours)
      real zhgt(0:mxvars)

      dimension freq(nws1,ndir1),wsfm(nws1,ndir1)

      real, allocatable,dimension(:,:) :: wsallo,wdallo,wsallp,wdallp
      integer, allocatable,dimension(:,:) :: ndateo,ndatep

      data cver,clev/'3.21','110225'/

c --- Configure output variables
      data dataset/'WIND.FRQ'/, dataver/'2.1'/
      data datamod/'CALWindRose FRQ File Format'/
      data ncomment/8/

c --- Character*320 filename string variables (metcomp.cmn)
c     fctr:  Control inp file name
c     fobs:  File name of input observed time series
c     fpre:  File name of input modeled time series
c     flst:  List file name (log file)
c     fid :  File name of station ID lists
c     frhs:  RHS of input timeseries files

c --- Set up the blank strings
      do k=1,80
         blnk80(k:k)=' '
      enddo
      blnk320(1:80)=blnk80
      blnk320(81:160)=blnk80
      blnk320(161:240)=blnk80
      blnk320(241:320)=blnk80

      chrdop=blnk80

      izero=0

c --- Initialize counters for calm periods
      ncalmo=0
      ncalmp=0
      njoint=0
      nvalid=0
      id_nochk=0
c --- Initialize counters for storage arrays
c --- Wind Speed
      nsioa=0
      nsbias=0
      nsrmse=0
      nsrmser=0
      nsioa24=0
      nsbias24=0
      nsgerr24=0
      nsrmse24=0
      nsrmser24=0
c --- Wind Direction
      ndbias=0
      ndbias24=0
      ndgerr24=0
c --- Temperature
      ntioa=0
      ntbias=0
      ntrmse=0
      ntrmser=0
      ntioa24=0
      ntbias24=0
      ntgerr24=0
      ntrmse24=0
      ntrmser24=0
c --- Specific Humidity
      nqioa=0
      nqbias=0
      nqrmse=0
      nqrmser=0
      nqioa24=0
      nqbias24=0
      nqgerr24=0
      nqrmse24=0
      nqrmser24=0
 
c ----------------------------------------------------------------
c --- Process control file
c ------------------------
      open(ictr,file='metcomp.inp',status='old')

c --- Determine version of control file
      icfver=1
      buff=blnk80
      read(ictr,'(a)') buff
      if(buff(1:16).EQ.'METCOMP.INP     ') then
         icfver=3
      else
c ---    Loop over all records and search for 'WINDROSE'
10       read(ictr,'(a)',end=11) buff
         iwr=INDEX(buff,'WINDROSE')
         if(iwr.GT.0) icfver=2
         goto 10
11       continue
         REWIND(ictr)
      endif

c --- Remove code segments used for older control files and
c --- only accept METCOMP.INP version 1.0 (for now)
      if(icfver.NE.3) then
         write(*,*)'HALTED:  METCOMP can not accept this early'
         write(*,*)'         version of the control file'
         stop
      endif

c --- Step through control file records up to WINDROSE section
c ------------------------------------------------------------
5     format(///,26x,'METCOMP OUTPUT SUMMARY',/,19x,'VERSION:  ',
     1       A12,' LEVEL:  ',A12///)

      if(icfver.EQ.3) then
c ---    First record already read
         read(ictr,'(a)')title
         ntl=LEN_TRIM(title)
c ---    Path for all output files
         fbpath=blnk320
         read(ictr,*)fbpath
         npath=LEN_TRIM(fbpath)
c ---    List filename (no path)
         flst=blnk320
         read(ictr,*)flst
         nlst=LEN_TRIM(flst)
c ---    Add path
         if(npath.GT.0) then
            flst=fbpath(1:npath)//flst(1:nlst)
            nlst=nlst+npath
         endif

         OPEN(ilog,file=flst(1:nlst),status='unknown')
c ---    Write banner to list file
         write(ilog,5) cver,clev
         write(ilog,'(a)')'Application: '//title(1:ntl)
         write(ilog,*)
         write(ilog,*)

c ---    Output file base name
         fbname=blnk320
         read(ictr,*)fbname
         nfb=LEN_TRIM(fbname)
c ---    Add path
         if(npath.GT.0) then
            fbas=fbpath(1:npath)//fbname(1:nfb)
         else
            fbas=fbname(1:nfb)
         endif
         ntbas=LEN_TRIM(fbas)
c ---    File name for list of station timeseries files
         fid=blnk320
         read(ictr,*) fid
         nid=LEN_TRIM(fid)
         write(ilog,*)'Station List File: '//fid(1:nid)
c ---    Base Time Zone
         azone='        '
         read(ictr,*)azone
         call UTCBASR(azone,xbtz)
         itmzbase=NINT(xbtz)
c ---    Processing Start Time
         cstart='              '
         read(ictr,*)cstart
         read(cstart(1:4),'(i4.4)') iyy1
         read(cstart(5:6),'(i2.2)') imm1
         read(cstart(7:8),'(i2.2)') idd1
         read(cstart(9:10),'(i2.2)') ihh1
         read(cstart(11:14),'(i4.4)') iss1
         call JULDAY(ilog,iyy1,imm1,idd1,ijj1)
         call TCLOCK(ilog,izero,iyy1,ijj1,ihh1,iss1)
         call GRDAY(ilog,iyy1,ijj1,imm1,idd1)
         idhr1=iyy1*100000+ijj1*100+ihh1
c ---    Processing Stop Time
         cstop='              '
         read(ictr,*)cstop
         read(cstop(1:4),'(i4.4)') iyy2
         read(cstop(5:6),'(i2.2)') imm2
         read(cstop(7:8),'(i2.2)') idd2
         read(cstop(9:10),'(i2.2)') ihh2
         read(cstop(11:14),'(i4.4)') iss2
         call JULDAY(ilog,iyy2,imm2,idd2,ijj2)
         call TCLOCK(ilog,izero,iyy2,ijj2,ihh2,iss2)
         call GRDAY(ilog,iyy2,ijj2,imm2,idd2)
         idhr2=iyy2*100000+ijj2*100+ihh2
c ---    Calm wind threshold
         read(ictr,*) wscalm
c ---    Flag of missing filter
         read(ictr,*) idfmis
      endif

c --- QA on times: require complete 24-hour periods so that the
c --- hour and second of the end-time must be the same as for the
c --- start time (but on a later day)
      ihhssss1=ihh1*10000+iss1
      ihhssss2=ihh2*10000+iss2
      iqatime=0
      if(ihhssss1.LT.ihhssss2) then
c ---    End hour-second moved up within day to match start
         iqatime=1
         ihh2=ihh1
         iss2=iss1
         ihhssss2=ihhssss1
         idhr2=iyy2*100000+ijj2*100+ihh2
      elseif(ihhssss1.GT.ihhssss2) then
c ---    End time must be moved up into previous day
         iqatime=2
         nhrinc=-24
         call INCR(ilog,iyy2,ijj2,ihh2,nhrinc)
         call GRDAY(ilog,iyy2,ijj2,imm2,idd2)
         ihh2=ihh1
         iss2=iss1
         ihhssss2=ihhssss1
         idhr2=iyy2*100000+ijj2*100+ihh2
      endif
c --- Now test date
      if(idhr1.GE.idhr2) then
         iqatime=3
      endif
c --- Report problem or adjustment
      if(iqatime.EQ.3) then
         write(*,*)'ERROR:  Statistics are based on full days so'
         write(*,*)'        that starting and ending times must'
         write(*,*)'        be at least 1 day apart'
         write(ilog,*)
         write(ilog,*)'ERROR:  Statistics are based on full days so'
         write(ilog,*)'        that starting and ending times must'
         write(ilog,*)'        be at least 1 day apart'
         write(ilog,*)'Start = ',cstart
         write(ilog,*)'End   = ',cstop
         stop
      elseif(iqatime.GT.0) then
c ---    Update stop-time string
         write(cstop(1:4),'(i4.4)') iyy2
         write(cstop(5:6),'(i2.2)') imm2
         write(cstop(7:8),'(i2.2)') idd2
         write(cstop(9:10),'(i2.2)') ihh2
         write(cstop(11:14),'(i4.4)') iss2
         write(*,*)'WARNING:  Statistics are based on full days so'
         write(*,*)'          the ending time has been reset'
         write(ilog,*)
         write(ilog,*)'WARNING:  Statistics are based on full days so'
         write(ilog,*)'          the ending time has been reset'
         write(ilog,*)'Start = ',cstart
         write(ilog,*)'End   = ',cstop
      endif

c --- Report selected information to list file
      write(ilog,*)
      write(ilog,*)'Wind speeds are set to ZERO if less than: ',
     &              wscalm,' m/s'
      if(wscalm.LT.0.0) then
        wscalm=wsclmd
        write(ilog,*)'Warning:  Negative calm speed threshold found'
        write(ilog,*)'          Set to Default Value = ',wsclmd
      endif
      if(idfmis.eq.1) then
         write(ilog,*)'Missing hours eliminated in frequency counts'
      else
         write(ilog,*)'Missing hours retained in frequency counts'
      endif
      write(ilog,*)'Base time zone:',itmzbase,azone
      write(ilog,'(a)')' Start of Period: '//cstart
      write(ilog,'(a18,i9.9,2x,i4.4)')
     &                 ' (YYYYJJJHH,SSSS) ',idhr1,iss1
      write(ilog,'(a)')' End   of Period: '//cstop
      write(ilog,'(a18,i9.9,2x,i4.4)')
     &                 ' (YYYYJJJHH,SSSS) ',idhr2,iss2


c --- Additional wind frequency windroses
c ---------------------------------------
      write(ilog,*)
      write(ilog,*)'WINDROSE Information'
c --- Skip section marker
      read(ictr,*)
      read(ictr,*)ntpd
      write(ilog,*)
     & 'Number of time periods for additional analyses:',ntpd
      irs=5
      if(ntpd.gt.0) then
         do i=1,ntpd
            read(ictr,*)(ntp(ii,i),ii=1,2)
            write(ilog,*)(ntp(ii,i),ii=1,2)
            irs=irs+1
            write(sname(irs),101)(ntp(ii,i),ii=1,2)
101         format('HR',i2.2,'-',i2.2)
c 101        format('TPD',i2.2,'_',i2.2)
            sname2(irs,1)='TPD'
            write(sname2(irs,2),111)(ntp(ii,i),ii=1,2)
111         format(i2,'_',i2)
         enddo
      endif
      read(ictr,*)nwsp
      write(ilog,*)
     &   'Number of WS ranges for additional analyses:',nwsp
      if(nwsp.gt.0) then
         do i=1,nwsp
            read(ictr,*)(wsp(ii,i),ii=1,2)
            write(ilog,*)(wsp(ii,i),ii=1,2)
            irs=irs+1
            write(sname(irs),102)(nint(wsp(ii,i)),ii=1,2)
 102        format('WSP',i2.2,'_',i2.2)
            sname2(irs,1)='WSP'
            write(sname2(irs,2),112)(nint(wsp(ii,i)),ii=1,2)
 112        format(i2.2,'_',i2.2)
         enddo
      endif

c --- Done with control file
      write(*,*)'Control File Processed'
      CLOSE(ictr)

c ----------------------------------------------------------------
c --- Open output files and write header lines
c --------------------------------------------

      write(ilog,*)
      write(ilog,*)'Files opened for output ---'

      fio1=fbas(1:ntbas)//'HourlyWindStats.dat'
      open(io1,file=fio1,status='unknown')
      write(io1,*)'Hourly Wind Speed and Direction'
      write(io1,*)'Wind speed .LE. ',wscalm,' m/s is treated as CALM'
      write(io1,1022)
      write(io1,1023)
      nt=index(fio1,' ')-1
      write(ilog,*)'Hourly Wind Speed and Direction         : ',
     &              fio1(1:nt)

      fio2=fbas(1:ntbas)//'DailyWindStats.dat'
      open(io2,file=fio2,status='unknown')
      write(io2,*)'Daily Wind Speed and Direction'
      write(io2,*)'Wind speed .LE. ',wscalm,' m/s is treated as CALM'
      write(io2,1022)
      write(io2,1023)
      nt=index(fio2,' ')-1
      write(ilog,*)'Daily  Wind Speed and Direction         : ',
     &              fio2(1:nt)

      fio3=fbas(1:ntbas)//'HourlyTeHuStats.dat'
      open(io3,file=fio3,status='unknown')
      write(io3,*)'Hourly Temperature and Specific Humidity'
      write(io3,1024)
      write(io3,1025)
      nt=index(fio3,' ')-1
      write(ilog,*)'Hourly Temperature and Specific Humidity: ',
     &              fio3(1:nt)

      fio4=fbas(1:ntbas)//'DailyTeHuStats.dat'
      open(io4,file=fio4,status='unknown')
      write(io4,*)'Daily Temperature and Specific Humidity'
      write(io4,1024)
      write(io4,1025)
      nt=index(fio4,' ')-1
      write(ilog,*)'Daily  Temperature and Specific Humidity: ',
     &              fio4(1:nt)

 1022 format('YYYY MM DD HH',
     &       '     IOA    Obs    Pred    Bias    Gross    ',
     &       'RMSE    RMSES   RMSEU    Obs    Pred    Bias   Gross ')
 1023 format('  -   -  -  -',
     &       '      -     m/s     m/s     m/s     m/s     ',
     &       ' m/s     m/s     m/s     deg     deg     deg     deg  ')

 1024 format('YYYY MM DD HH',
     &                    '     IOA    Obs   Pred     Bias    Gross',
     &                    '    RMSE   RMSES   RMSEU',
     &                    '     IOA    Obs   Pred     Bias    Gross',
     &                    '    RMSE   RMSES   RMSEU')
 1025 format('  -   -  -  -',
     &                    '      -      K      K        K       K ',
     &                    '      K       K       K  ',
     &                    '      -    g/kg   g/kg     g/kg    g/kg ',
     &                    '    g/kg    g/kg    g/kg ')

c ----------------------------------------------------------------
c --- Process station list for analysis
c -------------------------------------
c --- Filenmame list style is tied to ICFVAR=3
c --- Previous station ID list approach is not supported

      if(icfver.EQ.3) then
c ---    Read filename pairs from file
         OPEN(istn,file=fid(1:nid),status='old')
c ---    Header records
         char320=blnk320
         read(istn,'(a16)')char320
         if(char320(1:16).NE.'METCOMP.TSF.STN ') then
            write(*,*)'HALTED:  Invalid file type '//fid(1:nid)
            write(*,*)'         Expected: METCOMP.TSF.STN '
            write(*,*)'         Found   : ',char320(1:16)
            write(ilog,*)'HALTED:  Invalid file type '//fid(1:nid)
            write(ilog,*)'         Expected: METCOMP.TSF.STN '
            write(ilog,*)'         Found   : ',char320(1:16)
            stop
         endif
         read(istn,*)nskip
         do k=1,nskip
            read(istn,*)
         enddo
c ---    Documentation
         write(ilog,*)
         write(ilog,*)
         write(ilog,*)'Timeseries Files opened for input ---'
         write(ilog,*)
         write(ilog,*)'Type  I/O         File'
c ---    Data records
         kfil=0
22       fobs=blnk320
         fpre=blnk320
         read(istn,*,end=23)fpre,fobs
         nobs=LEN_TRIM(fobs)
         npre=LEN_TRIM(fpre)
         ngood=nobs+npre
         if(ngood.EQ.0) goto 23
         kfil=kfil+1
         nfil=MIN(kfil,MXFIL)
         fprea(nfil)=fpre
         fobsa(nfil)=fobs
c ---    Open current files
         iobs=in+2*(nfil-1)
         ipre=iobs+1
         write(ilog,*)' OBS ',iobs,'  ',fobs(1:nobs)
         write(ilog,*)' MOD ',ipre,'  ',fpre(1:npre)
         OPEN(iobs,file=fobs(1:nobs),status='old',action='read')
         OPEN(ipre,file=fpre(1:npre),status='old',action='read')
         goto 22

c ---    Done with file
23       CLOSE(istn)
         write(*,*)
c ---    Error if list is longer than dimension of arrays
         if(kfil.GT.MXFIL) then
            write(*,*)
            write(*,*)'ERROR:  Too many stations   = ',kfil
            write(*,*)'Maximum cannot exceed MXFIL = ',mxfil
            write(ilog,*)
            write(ilog,*)'ERROR:  Too many stations   = ',kfil
            write(ilog,*)'Maximum cannot exceed MXFIL = ',mxfil
            stop
         endif
      endif

c ----------------------------------------------------------------
c --- Allocate wsall and wdall arrays
      allocate(wsallo(mxhours,nfil),wdallo(mxhours,nfil))
      allocate(wsallp(mxhours,nfil),wdallp(mxhours,nfil))
      allocate(ndateo(mxhours,nfil),ndatep(mxhours,nfil))

      call ini17zero(nsbias,ndbias,ntbias,nqbias,nsrmse,nsrmser,
     &     ntrmse,ntrmser,nqrmse,nqrmser,nsgerr,ndgerr,ntgerr,
     &     nqgerr,nsioa,ntioa,nqioa)

      call ini17zero(nsbias24,ndbias24,ntbias24,nqbias24,
     &     nsgerr24,ndgerr24,ntgerr24,nqgerr24,
     &     nsrmse24,nsrmser24,ntrmse24,ntrmser24,
     &     nqrmse24,nqrmser24,nsioa24,ntioa24,nqioa24)

c ----------------------------------------------------------------
c --- Process header of timeseries files
c --------------------------------------
c --- Timeseries must be from METSERIES:
c --- MET_TS.DAT       Dataset Version 1.1
c --- TIMESERIES.TSF   Dataset Version 1.1
c --- TIMESERIES.TSF   Dataset Version 1.2
c --- TIMESERIES.TSF   Dataset Version 1.3
c --- Previous timeseries formats are not supported

      itmzo=-999
      itmzp=-999
      iqatser=0
      iqazone=0

      do k=1,nfil

c ---    J-Loop:  1=OBS, 2=PRED
         do j=1,2
c ---       File unit is i
            i=in + 2*(k-1) + (j-1)

c ---       Index for wind speed, direction, temperature,
c ---       and specific humidity
            kws(j,k)=0
            kwd(j,k)=0
            ktk(j,k)=0
            kqq(j,k)=0
            zhgt(0)=0.0

c ---       Test first header record to determine time series version
            itsver(j,k)=0
            read(i,'(2a16,a64)') tsernam,tserver,tsermod
            if(ADJUSTL(tsernam).EQ.'MET_TS.DAT      ') then
               if(ADJUSTL(tserver).EQ.'1.1             ') itsver(j,k)=1
            elseif(ADJUSTL(tsernam).EQ.'TIMESERIES.TSF  ') then
               if(ADJUSTL(tserver).EQ.'1.1             ')then
                  itsver(j,k)=2
               elseif(ADJUSTL(tserver).EQ.'1.2             ')then
                  itsver(j,k)=2
               elseif(ADJUSTL(tserver).EQ.'1.3             ')then
                  itsver(j,k)=3
               endif
            endif

            if(itsver(j,k).EQ.0) then
               write(ilog,*)' Error: Invalid Timeseries File ',k
               write(*,*)' Error: Invalid Timeseries File ',k
               iqatser=1
            endif

c ---       MET_TS.DAT       Dataset Version 1.1
            if(itsver(j,k).EQ.1) then
c ---          Skip comment records
               read(i,*) ncomm
               do n=1,ncomm
                  read(i,'(a)') comment
               enddo
c ---          Time Zone
               read(i,'(a)')azone8
c ---          Data description records
c ---          NOTE: units and missing value indicator are NOT USED!
               read(i,*) nvars(j,k)
               do n=1,nvars(j,k)
                  nn=MIN(n,mxvars)
                  read(i,'(2a12,f12.3)') avar(nn),aunits,xmissval
                  if(avar(nn).EQ.'WSPEED      ') kws(j,k)=nn
                  if(avar(nn).EQ.'WDIR        ') kwd(j,k)=nn
                  if(avar(nn).EQ.'TEMP        ') ktk(j,k)=nn
                  if(avar(nn).EQ.'SHUMID      ') kqq(j,k)=nn
               enddo
c ---          Expect 3 or more header records for title information
               read(i,*) ntitles
               if(ntitles.LT.3) then
                  write(*,*)'Wrong number of title lines in file'
                  write(*,*)'Found    ',ntitles
                  write(*,*)'Expected 3 or more'
                  stop
               endif
c ---          Read in and store three header lines
               do ii=1,3
                  read(i,'(a)')chrdop(ii,j,k)
               enddo
c ---          Skip all others
               do ii=4,ntitles
                  read(i,*)
               enddo
c ---          Modify 3rd title record for output
c ---          Drop information that does not pertain to winds
               klast=INDEX(chrdop(3,j,k),'(')-1
               hdr3=blnk80
               hdr3(1:klast)=chrdop(3,j,k)(1:klast)
               chrdop(3,j,k)=hdr3
            endif

c ---       TIMESERIES.TSF   Dataset Version 1.1 and 1.2
            if(itsver(j,k).EQ.2) then
c ---          Skip comment records
               read(i,*) ncomm
               do n=1,ncomm
                  read(i,'(a)') comment
               enddo
c ---          Time Zone
               read(i,'(a)')azone8
c ---          Data description records
c ---          NOTE: units and missing value indicator are NOT USED!
               read(i,*) nvars(j,k)
               do n=1,nvars(j,k)
                  nn=MIN(n,mxvars)
                  read(i,'(2a12,2f12.3)') avar(nn),aunits,xmissval,
     &                                    zhgt(nn)
                  if(avar(nn).EQ.'WSPEED      ') kws(j,k)=nn
                  if(avar(nn).EQ.'WDIR        ') kwd(j,k)=nn
                  if(avar(nn).EQ.'TEMP        ') ktk(j,k)=nn
                  if(avar(nn).EQ.'SHUMID      ') kqq(j,k)=nn
               enddo
c ---          Expect 2 or more header records for title information
               read(i,*) ntitles
               if(ntitles.LT.2) then
                  write(*,*)'Wrong number of title lines in file'
                  write(*,*)'Found    ',ntitles
                  write(*,*)'Expected 2 or more'
                  stop
               endif
c ---          Read in and store 2 header lines
               do ii=1,2
                  read(i,'(a)')chrdop(ii,j,k)
               enddo
c ---          Skip all others
               do ii=3,ntitles
                  read(i,*)
               enddo
c ---          Create 3rd title record for output
c ---          Place height for wind speed into 3rd title record
               chrdop(3,j,k)=blnk80
               write(chrdop(3,j,k),'(a8,f8.2,a2)')'Height: ',
     &                                   zhgt(kws(j,k)),' m'
            endif

c ---       TIMESERIES.TSF   Dataset Version 1.3
            if(itsver(j,k).EQ.3) then
c ---          Skip comment records
               read(i,*) ncomm
               do n=1,ncomm
                  read(i,'(a)') comment
               enddo
c ---          Expect 2 or more header records for title information
               read(i,*) ntitles
               if(ntitles.LT.2) then
                  write(*,*)'Wrong number of title lines in file'
                  write(*,*)'Found    ',ntitles
                  write(*,*)'Expected 2 or more'
                  stop
               endif
c ---          Read in and store 2 header lines
               do ii=1,2
                  read(i,'(a)')chrdop(ii,j,k)
               enddo
c ---          Skip all others
               do ii=3,ntitles
                  read(i,*)
               enddo
c ---          Skip map projection section
               read(i,*) pmap8
               if(pmap8.EQ.'NONE    ') then
                  nskip=0
               elseif(pmap8.EQ.'LL      ') then
                  nskip=2
               elseif(pmap8.EQ.'UTM     ') then
                  nskip=3
               else
                  nskip=4
               endif
               do n=1,nskip
                  read(i,*)
               enddo
c ---          Time Zone
               read(i,'(a)')azone8
               read(i,*)
               read(i,*)
c ---          Data description records
c ---          NOTE: units and missing value indicator are NOT USED!
               read(i,*) nvars(j,k)
               do n=1,nvars(j,k)
                  nn=MIN(n,mxvars)
                  read(i,'(2a12,5f12.3)') avar(nn),aunits,xmwt,
     &                                     xmissval,zhgt(nn),x,y
                  if(avar(nn).EQ.'WSPEED      ') kws(j,k)=nn
                  if(avar(nn).EQ.'WDIR        ') kwd(j,k)=nn
                  if(avar(nn).EQ.'TEMP        ') ktk(j,k)=nn
                  if(avar(nn).EQ.'SHUMID      ') kqq(j,k)=nn
               enddo
c ---          Create 3rd title record for output
c ---          Place height for wind speed into 3rd title record
               chrdop(3,j,k)=blnk80
               write(chrdop(3,j,k),'(a8,f8.2,a2)')'Height: ',
     &                                   zhgt(kws(j,k)),' m'
            endif

c ---       QA on time zone
            call UTCBASR(azone8,xtmz)
            itmz=NINT(xtmz)
c ---       Compute time-shift (integer hours) needed to convert times
c ---       in data files to the base time zone
            ihradd(j,k)=itmz-itmzbase
c            if(itmz.ne.itmzbase) then
c               write(ilog,*)' Error: Time Zone in File ',k
c               write(ilog,*)' File Zone, Base Zone: ',itmz,itmzbase
c               write(*,*)' Error: Time Zones do not match'
c               iqazone=1
c            endif
c            if(j.eq.1) then
c               if(itmzo.eq.-999) then
c                  itmzo=itmz
c               elseif(itmz.ne.itmzo) then
c                  write(*,301)itmz,itmzo
c                  write(ilog,301)itmz,itmzo
c                  iqazone=1
c               endif
c            else
c               if(itmzp.eq.-999) then
c                  itmzp=itmz
c               elseif(itmz.ne.itmzp) then
c                  write(*,302)itmz,itmzp
c                  write(ilog,302)itmz,itmzp
c                  iqazone=1
c               endif
c            endif
c 301        format(' Error - OBS Time zone not match',2i4)
c 302        format(' Error - PRED Time zone not match',2i4)


c ---       Inventory available measures
            lwind=.TRUE.
            ltmpk=.TRUE.
            lshum=.TRUE.
            if(kws(j,k).EQ.0) lwind=.FALSE.
            if(kwd(j,k).EQ.0) lwind=.FALSE.
            if(ktk(j,k).EQ.0) ltmpk=.FALSE.
            if(kqq(j,k).EQ.0) lshum=.FALSE.

            nproc=0
            if(.NOT.lwind) then
               write(ilog,*)'Wind data are not available'
               nproc=nproc+1
            endif
            if(.NOT.ltmpk) then
               write(ilog,*)'Temperature data are not available'
               nproc=nproc+1
            endif
            if(.NOT.lshum) then
               write(ilog,*)'Humidity data are not available'
               nproc=nproc+1
            endif
            if(nproc.EQ.3) then
               stop 'Program HALTED:  No Data Available'
            endif

         enddo
      enddo

      write(*,*)'Time-series Headers Processed'
      if(iqatser.NE.0) stop
c      if(iqazone.NE.0) stop

c --- Report time-zone shifts to log file
      write(ilog,*)
      write(ilog,*)'Time Zone adjustments for each input dataset'
      write(ilog,*)'Hours ADDED to data times to convert to ',azone
      j=1
      write(ilog,*)'Observed Data'
      write(ilog,*)'-------------'
      do k=1,nfil
         write(ilog,*) 'Base Time = Data plus ',ihradd(j,k),
     &                  '   '//TRIM(fobsa(k))
      enddo
      j=2
      write(ilog,*)'Predicted Data'
      write(ilog,*)'--------------'
      do k=1,nfil
         write(ilog,*) 'Base Time = Data plus ',ihradd(j,k),
     &                  '   '//TRIM(fprea(k))
      enddo
      write(ilog,*)
      write(ilog,*)


c ----------------------------------------------------------------
c --- Process data from timeseries files (24-hr blocks)
c ----------------------------------------------------------------
      ihro=0
      ihrp=0

      wsallo=fmiss
      wdallo=fmiss
      ndateo=-999
      wsallp=fmiss
      wdallp=fmiss
      ndatep=-999

c --- Establish step and initial period marker for first hour
      nsecdt=3600
      kyre(24)=iyy1
      kjule(24)=ijj1
      khre(24)=ihh1
      ksece(24)=iss1

c --- DAY LOOP
c ------------
5000  continue

c --- Compute the 24 target end-times 
      do nn=1,24
         nm1=nn-1
         if(nm1.EQ.0) nm1=24
         kyre(nn)=kyre(nm1)
         kjule(nn)=kjule(nm1)
         khre(nn)=khre(nm1)
         ksece(nn)=ksece(nm1)
         call INCRS(ilog,kyre(nn),kjule(nn),khre(nn),ksece(nn),nsecdt)
         kdahre(nn)=kyre(nn)*100000+kjule(nn)*100+khre(nn)
      enddo

      do k=1,nfil

c ---    Read observed day
c ------------------------
         j=1
         i=in + 2*(k-1) + (j-1)

         do ii=1,24
            wdo(k,ii)=fmiss
            wso(k,ii)=fmiss
            to(k,ii)=fmiss
            qo(k,ii)=fmiss
905         read(i,*,end=995) iy0,im0,id0,ih0,is0,
     &           iyr(ii),imon(ii),iday(ii),ihour(ii),isec,
     &           (xvar(kvar),kvar=1,nvars(j,k))
            call JULDAY(ilog,iyr(ii),imon(ii),iday(ii),ijul)
c ---       Time-shift to base time zone
            call INCR(ilog,iyr(ii),ijul,ihour(ii),ihradd(j,k))
c ---       Check for target date-time
            call TCLOCK(ilog,izero,iyr(ii),ijul,ihour(ii),isec)
            call GRDAY(ilog,iyr(ii),ijul,imon(ii),iday(ii))
            idahre=iyr(ii)*100000+ijul*100+ihour(ii)
            if(kdahre(ii).EQ.idahre .AND. ksece(ii).EQ.isec) then
               if(kwd(j,k).GT.0) wdo(k,ii)=xvar(kwd(j,k))
               if(kws(j,k).GT.0) wso(k,ii)=xvar(kws(j,k))
               if(ktk(j,k).GT.0) to(k,ii)=xvar(ktk(j,k))
               if(kqq(j,k).GT.0) qo(k,ii)=xvar(kqq(j,k))
            elseif(kdahre(ii).EQ.idahre .AND. ksece(ii).GT.isec) then
               goto 905
            elseif(kdahre(ii).GT.idahre) then
               goto 905
            else
               write(*,*)'ERROR:  Observed time not found'
               write(ilog,*)'ERROR:  Observed time not found'
               write(ilog,*)'        Timeseries file ',k
               write(ilog,*)'        Time = ',kdahre(ii),ksece(ii)
               write(ilog,*)'        Read = ',idahre,isec
               stop
            endif
         enddo
         i1end=ii-1

 995     continue
         i1end=ii-1
         if(i1end.lt.1) goto 996

C ---    Save data
C ---    time stamp - obs
         do ii=1,i1end
            if(k.eq.1) then
               ihro=ihro+1
               ip=ihro
            else
               ip=ihrobk+ii
            endif

            nyr=iyr(ii)
            nmn=imon(ii)
            ndy=iday(ii)
            nhr=ihour(ii)
            call timestamp(nyr,nmn,ndy,nhr,idate)
            ndateo(ip,k)=idate
            wsallo(ip,k)=wso(k,ii)
            wdallo(ip,k)=wdo(k,ii)

         enddo

 996     continue

c ---    Read predicted day
c -------------------------
         j=2
         i=in + 2*(k-1) + (j-1)

         do ii=1,24
            wdp(k,ii)=fmiss
            wsp(k,ii)=fmiss
            tp(k,ii)=fmiss
            qp(k,ii)=fmiss
907         read(i,*,end=997) iy0,im0,id0,ih0,is0,
     &           iyrp(ii),imonp(ii),idayp(ii),ihourp(ii),isec,
     &           (xvar(kvar),kvar=1,nvars(j,k))
            call JULDAY(ilog,iyrp(ii),imonp(ii),idayp(ii),ijul)
c ---       Time-shift to base time zone
            call INCR(ilog,iyrp(ii),ijul,ihourp(ii),ihradd(j,k))
c ---       Check for target date-time
            call TCLOCK(ilog,izero,iyrp(ii),ijul,ihourp(ii),isec)
            call GRDAY(ilog,iyrp(ii),ijul,imonp(ii),idayp(ii))
            idahre=iyrp(ii)*100000+ijul*100+ihourp(ii)
            if(kdahre(ii).EQ.idahre .AND. ksece(ii).EQ.isec) then
               if(kwd(j,k).GT.0) wdp(k,ii)=xvar(kwd(j,k))
               if(kws(j,k).GT.0) wsp(k,ii)=xvar(kws(j,k))
               if(ktk(j,k).GT.0) tp(k,ii)=xvar(ktk(j,k))
               if(kqq(j,k).GT.0) qp(k,ii)=xvar(kqq(j,k))
            elseif(kdahre(ii).EQ.idahre .AND. ksece(ii).GT.isec) then
               goto 907
            elseif(kdahre(ii).GT.idahre) then
               goto 907
            else
               write(*,*)'ERROR:  Modeled time not found'
               write(ilog,*)'ERROR:  Modeled time not found'
               write(ilog,*)'        Timeseries file ',k
               write(ilog,*)'        Time = ',kdahre(ii),ksece(ii)
               write(ilog,*)'        Read = ',idahre,isec
               stop
            endif
c            write(21,211) ! include T2 and Q2
c     &           iyr(ii),imon(ii),iday(ii),ihour(ii),
c     &           wdp(k,ii),wsp(k,ii),tp(k,ii),qp(k,ii)

         enddo
         i2end=ii-1

 997     continue
         i2end=ii-1
         if(i2end.lt.1) goto 998

C ---    time stamp - pred
         do ii=1,i2end
            if(k.eq.1) then
               ihrp=ihrp+1
               ip=ihrp
            else
               ip=ihrpbk+ii
            endif

            nyr=iyrp(ii)
            nmn=imonp(ii)
            ndy=idayp(ii)
            nhr=ihourp(ii)
            call timestamp(nyr,nmn,ndy,nhr,idate)
            ndatep(ip,k)=idate
            wsallp(ip,k)=wsp(k,ii)
            wdallp(ip,k)=wdp(k,ii)

c            write(80,810)k,ip,ndatep(ip,k),wsallp(ip,k),wdallp(ip,k)
c 810        format(2i6,2x,i10.10,2f12.2)
         enddo

 998     continue
         ihrobk=ihro-i1end
         ihrpbk=ihrp-i2end

      enddo

c      print *,ip,ndateo(ip,1),ndatep(ip,1),i1end,i2end

      if(i1end.lt.24 .or. i2end.lt.24) goto 999
         
c --- Do the statistics (hours and day)
      call STATDAY(nfil)

c --- Write timeseries of stats to files
c --- Hourly
      do k=1,24
         write(io1,1026) iyr(k),imon(k),iday(k),ihour(k),
     &              sioa(k),wsom(k),wspm(k),sbias(k),
     &              sgerr(k),srmse(k),srmses(k),srmseu(k),
     &              wdom(k),wdpm(k),dbias(k),dgerr(k)
         write(io3,1027) iyr(k),imon(k),iday(k),ihour(k),
     &              tioa(k),tom(k),tpm(k),tbias(k),
     &              tgerr(k),trmse(k),trmses(k),trmseu(k),
     &              qioa(k),qom(k),qpm(k),qbias(k),
     &              qgerr(k),qrmse(k),qrmses(k),qrmseu(k)
      enddo

c --- Daily
      k=1
         write(io2,1026) iyr(k),imon(k),iday(k),ihour(k),
     &              sioa24,wsom24,wspm24,sbias24,
     &              sgerr24,srmse24,srmses24,srmseu24,
     &              wdom24,wdpm24,dbias24,dgerr24
         write(io4,1027) iyr(k),imon(k),iday(k),ihour(k),
     &              tioa24,tom24,tpm24,tbias24,
     &              tgerr24,trmse24,trmses24,trmseu24,
     &              qioa24,qom24,qpm24,qbias24,
     &              qgerr24,qrmse24,qrmses24,qrmseu24

 1026 format(i4,3i3,f8.2,2f7.1,2f9.2,3f8.2,2f8.1,2f8.2)
 1027 format(i4,3i3,2(f8.2,2f7.1,2f9.2,3f8.2))

c --- Update storage arrays with non-missing results
      call STORE

c --- Do next 24 hours if a complete day was processed
      if(i1end.eq.24 .and. i2end.eq.24) then
c ---    Check last end-time to process (idhr2,iss2)
         if(kdahre(24).EQ.idhr2 .AND. ksece(24).EQ.iss2) then
c ---       Done
         else
c ---       Next Day
            goto 5000
         endif
      endif

999   continue

C --- Finished reading all time series
c --- Close all timeseries files
      do k=1,nfil
         iobs=in+2*(k-1)
         ipre=iobs+1
         close(iobs)
         close(ipre)
      enddo

C --- Check data
      nptso=ihro
      nptsp=ihrp

      if(iecho.ne.1) goto 880
      nn=min(5,nfil)
      do ihr=1,nptso
         write(30,210)ndateo(ihr,1)
     &     ,(wsallo(ihr,jj),wdallo(ihr,jj),jj=1,nn)
      enddo

      do ihr=1,nptsp
         write(31,210)ndatep(ihr,1)
     &     ,(wsallp(ihr,jj),wdallp(ihr,jj),jj=1,nn)
      enddo
 210  format(i10.10,20f9.1)

 880  continue

c -------------------------------------------------------------------
C --- Wind frequency
c -------------------------------------------------------------------
      npts=min(nptso,nptsp)
      print *,'Hours Obs,Model: ',nptso,nptsp

      write(ilog,*)
      write(ilog,*)'Wind Frequency Distribution ---'
      write(ilog,*)

C --- Check time and Filter missing
      do k=1,nfil
         do ihr=1,npts
            idate1=ndateo(ihr,k)
            idate2=ndatep(ihr,k)

            if(id_nochk.eq.1) goto 215
            if(idate1.ne.idate2) then
               write(*,212)ihr,idate1,idate2
               write(ilog,212)ihr,idate1,idate2
 212           format(' Error: Time mismatch:',i5,2i12)
               stop
            endif
 215        continue

            if(idfmis.eq.1) then
               sso=wsallo(ihr,k)
               ddo=wdallo(ihr,k)
               ssp=wsallp(ihr,k)
               ddp=wdallp(ihr,k)

               if(sso.eq.fmiss .or. ssp.eq.fmiss .or. 
     &            ddo.eq.fmiss .or. ddp.eq.fmiss) then
                  wsallo(ihr,k)=fmiss
                  wdallo(ihr,k)=fmiss
                  wsallp(ihr,k)=fmiss
                  wdallp(ihr,k)=fmiss
               endif
            endif

         enddo
      enddo

c --- Wind freq for paired time series
      do 6000 k=1,nfil

c ---    Generate output filenames
c --------------------------------
         fqro=blnk320
         fqrp=blnk320
         ftbo=blnk320
         ftbp=blnk320


         if(icfver.EQ.3) then
c ---       Observed
            i2=LEN_TRIM(fobsa(k))
c ---       Exclude path from name
            i1=INDEX(fobsa(k),'\',back=.TRUE.)+1
c ---       Add output path
            fqro=fbpath(1:npath)//fobsa(k)(i1:i2)
            last=LEN_TRIM(fqro)
c ---       Drop in processing period
            iutc=INDEX(fqro,'UTC')
            if(iutc.GT.0) then
               i1=iutc-1
               i2=iutc+38
               fqro(i1:i2)='_'//azone//'_'//cstart//'-'//cstop
            else
               nt=INDEX(fqro,'.',back=.true.)
               i1=nt
               i2=nt+39
               fqro(i1:i2)='_'//azone//'_'//cstart//'-'//cstop//'.'
            endif
c ---       Extension
            nt=INDEX(fqro,'.',back=.true.)
            ftbo=fqro(1:nt)//'tab'
            fqro=ftbo(1:nt)//'frq'
c ---       Modeled
            i2=LEN_TRIM(fprea(k))
c ---       Exclude path from name
            i1=INDEX(fprea(k),'\',back=.TRUE.)+1
c ---       Add output path
            fqrp=fbpath(1:npath)//fprea(k)(i1:i2)
            last=LEN_TRIM(fqrp)
c ---       Drop in processing period
            iutc=INDEX(fqrp,'UTC')
            if(iutc.GT.0) then
               i1=iutc-1
               i2=iutc+38
               fqrp(i1:i2)='_'//azone//'_'//cstart//'-'//cstop
            else
               nt=INDEX(fqrp,'.',back=.true.)
               i1=nt
               i2=nt+39
               fqrp(i1:i2)='_'//azone//'_'//cstart//'-'//cstop//'.'
            endif
c ---       Extension
            nt=INDEX(fqrp,'.',back=.true.)
            ftbp=fqrp(1:nt)//'tab'
            fqrp=ftbp(1:nt)//'frq'
         endif


      do 6000 j=1,2   ! obs/pre

c ---    Generate frequencies
         wssbkp=fmiss
         wdsbkp=fmiss

         nwdrs=0

         do ihr=1,npts            
            if(j.eq.1) then
               ss=wsallo(ihr,k)
               dd=wdallo(ihr,k)
            else
               ss=wsallp(ihr,k)
               dd=wdallp(ihr,k)
            endif
            wssbkp(ihr)=ss
            wdsbkp(ihr)=dd
         enddo

C ----   Annual
         nodata=0
         wss=wssbkp
         wds=wdsbkp
         call wdfreq(wss,wds,freq,wsfm,npts,nodata)
         nwdrs=nwdrs+1

         if(nodata.eq.1) then
            write(ilog,312)k,j
 312        format(' No Data at Site/OBS_MOD:',2i8) 
            goto 6000
         endif

c ---    Set progress marker
         if(j.eq.1) then
            print *,' Processing Site: ',k
            write(ilog,*)' Processing Site: ',k
         endif

         if(j.eq.1) then
c ---       Observed
            open(ifq,file=fqro,status='unknown',action='write')
            open(itb,file=ftbo,status='unknown',action='write')
         else
c ---       Modeled
            open(ifq,file=fqrp,status='unknown',action='write')
            open(itb,file=ftbp,status='unknown',action='write')
         endif

c ---    Write header records to FRQ file:
         write(ifq,'(2a16,a64)') dataset,dataver,datamod
         write(ifq,'(i4)') ncomment
         write(ifq,'(a)')'Created by METCOMP   '//cver//clev
         write(ifq,'(a)') chrdop(1,j,k)
         write(ifq,'(a)') chrdop(2,j,k)
         write(ifq,'(a)') chrdop(3,j,k)

         write(itb,'(a)')'Created by METCOMP   '//cver//clev
         write(itb,'(a)') chrdop(1,j,k)
         write(itb,'(a)') chrdop(2,j,k)
         write(itb,'(a)') chrdop(3,j,k)

c ---    Note: In order to write out calms as eg. 1.0E-05, the 
c              1pe9.1 format descriptor was used which moves all
c              decimal places right by one. This works for the
c              e9.1 format, but causes an equivalent x10 to the
c              values being written using f5.1, which is why 
c              wscls(i) is divided by 10 when written out here
c              This only affects this write statement
c           write(ifq,1120) wscalm,(wscls(i)/10,i=1,nws-1)
         write(ifq,1120) wscalm,wscalm/10,(wscls(i)/10,i=1,nws-1)
 1120    format('Wind Speed Classes (m/s):',(1pe9.1),
     &          6(',',f5.1))
c ---    Beg/end times
         if(j.eq.1) then
            idateb=ndateo(1,k)
            idatee=ndateo(npts,k)
         else
            idateb=ndatep(1,k)
            idatee=ndatep(npts,k)
         endif

         call stamptime(idateb,iyrb,imnb,idyb,ihrb)
         call stamptime(idatee,iyre,imne,idye,ihre)

         write(ifq,1130)iyrb,imnb,idyb,ihrb,iyre,imne,idye,ihre
         write(itb,1130)iyrb,imnb,idyb,ihrb,iyre,imne,idye,ihre
 1130    format(i4,3i3,3x,i4,3i3)

C ---    Additional 2 comment lines
         zone=float(itmz)
         call BASRUTC(zone,azone)
         write(ifq,1031)azone
         write(itb,1031)azone
 1031    format(a8)
         if(idfmis.eq.1) then
            if(j.eq.1) then
               write(ifq,1032)trim(fprea(k))
               write(itb,1032)trim(fprea(k))
            else
               write(ifq,1032)trim(fobsa(k))
               write(itb,1032)trim(fobsa(k))
            endif
         else
            write(ifq,1033)
            write(itb,1033)
         endif
 1032    format('Missing filtered - Companion File: ',a)
 1033    format('Missing not filtered' )

         idm=1
         call freqout(freq,wsfm,npts,idm)

c ---    Four seasons
         do isn=1,4
            wds=fmiss
            wss=fmiss

            ip=0
            do ihr=1,npts
               if(j.eq.1) then
                  idate=ndateo(ihr,k)
               else
                  idate=ndatep(ihr,k)
               endif

               call stamptime(idate,iyr1,imn1,idy1,ihh1)               
               ids=msn(imn1)
               if(ids.eq.isn) then
                  ip=ip+1
                  wds(ip)=wdsbkp(ihr)
                  wss(ip)=wssbkp(ihr)
               endif
            enddo

            ipts=ip
            nodata=0

            call wdfreq(wss,wds,freq,wsfm,ipts,nodata)
            nwdrs=nwdrs+1
            if(nodata.eq.0) then
               idm=isn+1
               call freqout(freq,wsfm,ipts,idm)
            else
               if(j.EQ.1)
     &         write(ilog,*)'  - No Obs. Data for Season ',isn
               if(j.EQ.2)
     &         write(ilog,*)'  - No Mod. Data for Season ',isn
            endif

         enddo

c        Additional time periods
         if(ntpd.le.0) goto 2000
         do itpd=1,ntpd
            iprd1=ntp(1,itpd)
            iprd2=ntp(2,itpd)

            irev=0
            if(iprd1.gt.iprd2) irev=1

            ip=0
            wds=fmiss
            wss=fmiss

            do ihr=1,npts
               if(j.eq.1) then
                  idate=ndateo(ihr,k)
               else
                  idate=ndatep(ihr,k)
               endif

               call stamptime(idate,iyr1,imn1,idy1,ihh1)
               iprd=ihh1

               if(irev.eq.0) then
c                  if(iprd.ge.iprd1 .and. iprd.lt.iprd2) then
                  if(iprd.ge.iprd1 .and. iprd.le.iprd2) then
                     ip=ip+1
                     wds(ip)=wdsbkp(ihr)
                     wss(ip)=wssbkp(ihr)
                  endif
               else
c                  if(iprd.ge.iprd1 .or. iprd.lt.iprd2) then
                  if(iprd.ge.iprd1 .or. iprd.le.iprd2) then
                     ip=ip+1
                     wds(ip)=wdsbkp(ihr)
                     wss(ip)=wssbkp(ihr)
                  endif
               endif
            enddo

            ipts=ip
            nodata=0

            call wdfreq(wss,wds,freq,wsfm,ipts,nodata)
            nwdrs=nwdrs+1

            if(nodata.eq.0) then
               idm=itpd+5
               call freqout(freq,wsfm,ipts,idm)
            else
               if(j.EQ.1)
     &         write(ilog,*)'  - No Obs. Data for Time Period ',itpd
               if(j.EQ.2)
     &         write(ilog,*)'  - No Mod. Data for Time Period ',itpd
            endif
         enddo

c        Additional wind speed ranges
 2000    continue
         if(nwsp.le.0) goto 2500

         do iwsp=1,nwsp
            ss1=wsp(1,iwsp)
            ss2=wsp(2,iwsp)

            ip=0
            wds=fmiss
            wss=fmiss

            do ihr=1,npts
               ss=wssbkp(ihr)
c               if(ss.ge.ss1 .and. ss.lt.ss2) then
               if(ss.ge.ss1 .and. ss.le.ss2) then
                  ip=ip+1
                  wds(ip)=wdsbkp(ihr)
                  wss(ip)=wssbkp(ihr)
               endif
            enddo

            ipts=ip
            nodata=0

            call wdfreq(wss,wds,freq,wsfm,ipts,nodata)
            nwdrs=nwdrs+1

            if(nodata.eq.0) then
               idm=iwsp+5+ntpd
               call freqout(freq,wsfm,ipts,idm)
            else
               if(j.EQ.1)
     &         write(ilog,*)'  - No Obs. Data for Speed Range ',iwsp
               if(j.EQ.2)
     &         write(ilog,*)'  - No Mod. Data for Speed Range ',iwsp
            endif
         enddo

c        Finish windrose part for one time series
 2500    continue

         close(ifq)
         close(itb)

 6000 continue                  ! Finish frequency

c --- Summarize statistics
c --- Repeat column headers for summary section
      write(io1,'(//)')
      write(io1,*)'Hourly Wind Speed and Direction Summary'
      write(io1,*)'Wind speed .LE. ',wscalm,' m/s is treated as CALM'
      write(io1,*)
      write(io1,1122)
      write(io1,1123)
      write(io2,'(//)')
      write(io2,*)'Daily Wind Speed and Direction Summary'
      write(io2,*)'Wind speed .LE. ',wscalm,' m/s is treated as CALM'
      write(io2,*)
      write(io2,1122)
      write(io2,1123)
      write(io3,'(//)')
      write(io3,*)'Hourly Temperature and Specific Humidity Summary'
      write(io3,*)
      write(io3,1124)
      write(io3,1125)
      write(io4,'(//)')
      write(io4,*)'Daily Temperature and Specific Humidity Summary'
      write(io4,*)
      write(io4,1124)
      write(io4,1125)

      write(ilog,'(////)')
      write(ilog,'(a)')'                 DAILY SUMMARY'
      write(ilog,*)
      write(ilog,'(a)')title(1:ntl)
      write(ilog,*)'Wind speeds are set to ZERO if less than: ',
     &              wscalm,' m/s'
      write(ilog,*)'Number of observation stations: ',nfil

      call SUMMARY

c --- Report CALM period tally to end of hourly and daily wind reports
      do k=1,3
         if(k.EQ.1) io=io1
         if(k.EQ.2) io=io2
         if(k.EQ.3) io=ilog
         write(io,'(//)') 
         write(io,*) 'Calm period counts (station-hours)'
         write(io,*) 'Number of observed   calms: ',ncalmo
         write(io,*) 'Number of predicted  calms: ',ncalmp
         write(io,*) 'Number of concurrent calms: ',njoint
         write(io,*) 'Non-missing  station-hours: ',nvalid
      enddo

 1122 format('             ',
     &       '     IOA    Obs    Pred    Bias    Gross    ',
     &       'RMSE    RMSES   RMSEU    Obs    Pred    Bias   Gross ')
 1123 format('             ',
     &       '      -     m/s     m/s     m/s     m/s     ',
     &       ' m/s    /RMSE   /RMSE    deg     deg     deg     deg  ')


 1124 format('             ',
     &                    '     IOA    Obs   Pred     Bias    Gross',
     &                    '    RMSE   RMSES   RMSEU',
     &                    '     IOA    Obs   Pred     Bias    Gross',
     &                    '    RMSE   RMSES   RMSEU')
 1125 format('             ',
     &                    '      -      K      K        K       K ',
     &                    '      K     /RMSE   /RMSE',
     &                    '      -    g/kg   g/kg     g/kg    g/kg ',
     &                    '    g/kg   /RMSE   /RMSE ')

      write(*,*)'Processing Completed'

      deallocate(wsallo,wdallo,ndateo)
      deallocate(wsallp,wdallp,ndatep)

      stop
      end


c----------------------------------------------------------------------
c --- CALPUFF System Utilities
      include 'calutils.for
c----------------------------------------------------------------------


c----------------------------------------------------------------------
      BLOCK DATA
c----------------------------------------------------------------------
c
c --- METCOMP    Version: 3.21     Level: 070207            BLOCK DATA
c                D. Strimaitis
c
c----------------------------------------------------------------------
c
c --- Include parameter statements
      include 'metcomp.par'
c
c --- Include common blocks
      include 'metcomp.cmn'

      data wscls/1.8,3.3,5.4,8.5,10.8,100.0/
      data sdir/'N','NNE','NE','ENE','E','ESE','SE','SSE',
     &          'S','SSW','SW','WSW','W','WNW','NW','NNW','SUM'/
      data (sname(i),i=1,5)/'Annual(J-D)','Winter(D-J-F)',
     &      'Spring(M-A-M)','Summer(J-J-A)','Autumn(S-O-N)'/
      data ((sname2(i,j),i=1,5),j=1,2)/'Annual','Winter','Spring',
     &     'Summer','Autumn','(J-D)','(D-J-F)','(M-A-M)',
     &     '(J-J-A)','(S-O-N)'/

      data msn/1,1,2,2,2,3,3,3,4,4,4,1/

      end

c----------------------------------------------------------------------
      subroutine statday(nsite)
c ---------------------------------------------------------------------
c
c --- METCOMP    Version: 3.21       Level: 060630              STATDAY
c ---            D. Strimaitis
c
c --- Prepare arrays (no missing values) and call STATS to compute 
c     hourly and daily performance statistics.
c
c --- INPUTS:
c             NSITE - integer    - Number of elements in arrays each
c                                  hour
c --- OUTPUT:
c
c --- UPDATES:
c
c --- Version 1.0, Level 060612 to Version 1.1, Level 060630 (DGS)
c
c       1.  Add CALM threshold wind speed value that converts very low
c           wind speed occurrences (observed or modeled) to "calms".
c           The statistis for wind speed will use 0.0 m/s for such
c           cases, and the corresponding wind direction will be
c           excluded from the wind direction statistics.  Note that a
c           calm threshold of 0.0 m/s implicitly removes this screen.
c
c       2.  Tally counts of calm occurrences as total number of periods
c           observed, total number of periods modeled, and total number
c           of coincident (space/time) calm periods.
c
c
c --- STATDAY called by:  main
c --- STATDAY calls:      STATS, VECAVG
c ---------------------------------------------------------------------
c
      include 'metcomp.par'
      parameter (mxnum=mxhr1*mxsite)

      include 'metcomp.cmn'

c --- Local arrays (observed, predicted)
      real os(mxnum),od(mxnum),ot(mxnum),oq(mxnum)
      real ps(mxnum),pd(mxnum),pt(mxnum),pq(mxnum)
c --- Arrays with no calm hours
      real osnc(mxnum),odnc(mxnum)
      real psnc(mxnum),pdnc(mxnum)

      fdum=dum
c --- Hourly stats
      do i = 1,mxhr1

c ---    Assign data to local arrays of non-missing values
         nwind=0
         nocalm=0
         ntemp=0
         nshum=0
         do j=1,nsite
            if(wso(j,i).LT.9990.0 .AND. wdo(j,i).LT.9990.0) then
               nvalid=nvalid+1
               nwind=nwind+1
               os(nwind)=wso(j,i)
               od(nwind)=wdo(j,i)
               ps(nwind)=wsp(j,i)
               pd(nwind)=wdp(j,i)
c ---          Apply explicit calm threshold
c ---          (force 'unresolvable' speeds to zero)
               if(os(nwind).LE.wscalm) os(nwind)=0.0
               if(ps(nwind).LE.wscalm) ps(nwind)=0.0
c ---          Arrays without calms
               if(wso(j,i).GT.wscalm .AND. wsp(j,i).GT.wscalm) then
                  nocalm=nocalm+1
                  osnc(nocalm)=wso(j,i)
                  odnc(nocalm)=wdo(j,i)
                  psnc(nocalm)=wsp(j,i)
                  pdnc(nocalm)=wdp(j,i)
               else
                  if(wso(j,i).LE.wscalm) ncalmo=ncalmo+1
                  if(wsp(j,i).LE.wscalm) ncalmp=ncalmp+1
                  if(wso(j,i).LE.wscalm .AND. 
     &               wsp(j,i).LE.wscalm) njoint=njoint+1
               endif
            endif
            if(to(j,i).LT.9990.0 .AND. tp(j,i).LT.9990.0) then
               ntemp=ntemp+1
               ot(ntemp)=to(j,i)
               pt(ntemp)=tp(j,i)
            endif
            if(qo(j,i).LT.9990.0 .AND. qp(j,i).LT.9990.0) then
               nshum=nshum+1
               oq(nshum)=qo(j,i)
               pq(nshum)=qp(j,i)
            endif
         enddo

c ---    Wind speed
         if(nwind.GT.0) then
            call stats(os,ps,nwind,'----',wsom(i),wspm(i),sbias(i),
     &           sgerr(i),ra,rb,srmse(i),srmses(i),srmseu(i),sioa(i))
         else
            wsom(i)=dum
            wspm(i)=dum
            sbias(i)=dum
            sgerr(i)=dum
            srmse(i)=dum
            srmses(i)=dum
            srmseu(i)=dum
            sioa(i)=dum
         endif

c ---    Wind direction
c ---    Mean direction uses a vector average
         if(nwind.GT.0 .AND. nocalm.GT.0) then
            call vecavg(os,od,nwind,wdom(i))
            call vecavg(ps,pd,nwind,wdpm(i))
            call stats(odnc,pdnc,nocalm,'WDIR',fdum,fdum,dbias(i),
     &           dgerr(i),ra,rb,fdum,fdum,fdum,fdum)
         else
            wdom(i)=dum
            wdpm(i)=dum
            dbias(i)=dum
            dgerr(i)=dum
         endif

c ---    Temperature
         if(ntemp.GT.0) then
            call stats(ot,pt,ntemp,'----',tom(i),tpm(i),tbias(i),
     &           tgerr(i),ra,rb,trmse(i),trmses(i),trmseu(i),tioa(i))
         else
            tom(i)=dum
            tpm(i)=dum
            tbias(i)=dum
            tgerr(i)=dum
            trmse(i)=dum
            trmses(i)=dum
            trmseu(i)=dum
            tioa(i)=dum
         endif

c ---    Humidity
         if(nshum.GT.0) then
            call stats(oq,pq,nshum,'----',qom(i),qpm(i),qbias(i),
     &           qgerr(i),ra,rb,qrmse(i),qrmses(i),qrmseu(i),qioa(i))
         else
            qom(i)=dum
            qpm(i)=dum
            qbias(i)=dum
            qgerr(i)=dum
            qrmse(i)=dum
            qrmses(i)=dum
            qrmseu(i)=dum
            qioa(i)=dum
         endif

      enddo

c --- Daily stats
c --- Assign data to local arrays of non-missing values
      nwind=0
      nocalm=0
      ntemp=0
      nshum=0

      do i=1,mxhr1
         do j=1,nsite
            if(wso(j,i).LT.9990.0 .AND. wdo(j,i).LT.9990.0) then
               nwind=nwind+1
               os(nwind)=wso(j,i)
               od(nwind)=wdo(j,i)
               ps(nwind)=wsp(j,i)
               pd(nwind)=wdp(j,i)
c ---          Apply explicit calm threshold
c ---          (force 'unresolvable' speeds to zero)
               if(os(nwind).LE.wscalm) os(nwind)=0.0
               if(ps(nwind).LE.wscalm) ps(nwind)=0.0
c ---          Arrays without calms
               if(wso(j,i).GT.wscalm .AND. wsp(j,i).GT.wscalm) then
                  nocalm=nocalm+1
                  osnc(nocalm)=wso(j,i)
                  odnc(nocalm)=wdo(j,i)
                  psnc(nocalm)=wsp(j,i)
                  pdnc(nocalm)=wdp(j,i)
               endif
            endif
            if(to(j,i).LT.9990.0 .AND. tp(j,i).LT.9990.0) then
               ntemp=ntemp+1
               ot(ntemp)=to(j,i)
               pt(ntemp)=tp(j,i)
            endif
            if(qo(j,i).LT.9990.0 .AND. qp(j,i).LT.9990.0) then
               nshum=nshum+1
               oq(nshum)=qo(j,i)
               pq(nshum)=qp(j,i)
            endif
         enddo
      enddo

c --- Wind speed
      if(nwind.GT.0) then
         call stats(os,ps,nwind,'----',wsom24,wspm24,sbias24,sgerr24,
     &              ra,rb,srmse24,srmses24,srmseu24,sioa24)
      else
         wsom24=dum
         wspm24=dum
         sbias24=dum
         sgerr24=dum
         srmse24=dum
         srmses24=dum
         srmseu24=dum
         sioa24=dum
      endif

c --- Wind direction
c --- Mean direction uses a vector average
      if(nwind.GT.0 .AND. nocalm.GT.0) then
         call vecavg(os,od,nwind,wdom24)
         call vecavg(ps,pd,nwind,wdpm24)
         call stats(odnc,pdnc,nocalm,'WDIR',fdum,fdum,dbias24,dgerr24,
     &              ra,rb,fdum,fdum,fdum,fdum)
      else
         wdom24=dum
         wdpm24=dum
         dbias24=dum
         dgerr24=dum
      endif

c --- Temperature
      if(ntemp.GT.0) then
         call stats(ot,pt,ntemp,'----',tom24,tpm24,tbias24,tgerr24,
     &              ra,rb,trmse24,trmses24,trmseu24,tioa24)
      else
         tom24=dum
         tpm24=dum
         tbias24=dum
         tgerr24=dum
         trmse24=dum
         trmses24=dum
         trmseu24=dum
         tioa24=dum
      endif

c --- Humidity
      if(nshum.GT.0) then
         call stats(oq,pq,nshum,'----',qom24,qpm24,qbias24,qgerr24,
     &              ra,rb,qrmse24,qrmses24,qrmseu24,qioa24)
      else
         qom24=dum
         qpm24=dum
         qbias24=dum
         qgerr24=dum
         qrmse24=dum
         qrmses24=dum
         qrmseu24=dum
         qioa24=dum
      endif


      return
      end

c----------------------------------------------------------------------
      subroutine vecavg(ws,wd,n,wdmean)
c ---------------------------------------------------------------------
c
c --- METCOMP    Version: 3.21       Level: 060612              VECAVG
c ---            D. Strimaitis
c
c --- Compute vector average direction
c
c --- INPUTS:
c                WS - real array - Set of speeds
c                WD - real array - Set of directions
c                 N - integer    - Number of elements in arrays
c
c --- OUTPUT:
c            WDMEAN - real       - Vector-mean direction
c
c --- VECAVG called by:  STATDAY
c --- VECAVG calls:      none
c ---------------------------------------------------------------------
c
      include 'metcomp.par'

      real ws(n),wd(n)

      usum=0.0
      vsum=0.0
      nsum=0

c --- Loop over data
      do k=1,n
         if(ws(k).GT.0.0) then
            nsum=nsum+1
c ---       Convert to U,V components and update sums
            usum=usum-ws(k)*sin(wd(k)*deg2rad)
            vsum=vsum-ws(k)*cos(wd(k)*deg2rad)
         endif
      enddo

      if(nsum.EQ.0) then
         wdmean=dum
      else
         xnum=FLOAT(nsum)
         umean=usum/xnum
         vmean=vsum/xnum
         flow=rad2deg*ATAN2(umean,vmean)
         wdmean=flow+180.
         if(wdmean.GT.360.) wdmean=wdmean-360.
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine stats(xobs,ypre,n,wdflag,xomean,ypmean,bias,gross,
     &                 ra,rb,rmse,rmses,rmseu,rioa)
c ---------------------------------------------------------------------
c
c --- METCOMP    Version: 3.21       Level: 060612                STATS
c ---            D. Strimaitis
c
c --- Compute comparision statistics for one set of prediction - 
c     observation residuals.  The data pairs for the residuals must
c     be valid (no missing values here).
c
c     Special treatment of WIND DIRECTION:  +/-180 range-check
c     is imposed on residual, and this residual is placed in the
c     predicted variable.  The observed is set to zero.  Only the bias
c     and gross error stats are returned.  The direction means are
c     computed elsewhere as vector averages.
c
c --- INPUTS:
c              XOBS - real array - Set of observations
c              YPRE - real array - Set of predictions
c                 N - integer    - Number of elements in arrays
c            WDFLAG - char*4     - Wind direction flag 'WDIR'
c
c --- OUTPUT:
c            XOMEAN - real       - Mean observed
c            YPMEAN - real       - Mean predicted
c              BIAS - real       - Bias in residual (P-O)
c             GROSS - real       - Gross error in residual |P-O|
c                RA - real       - Regression intercept
c                RB - real       - Regression slope
c              RMSE - real       - Root mean square error
c             RMSES - real       - RMSE (systematic)
c             RMSEU - real       - RMSE (unsystematic)
c                                  (RMSE^2=RMSES^2+RMSEU^2)
c              RIOA - real       - Index of agreement
c
c --- STATS called by:  STATDAY
c --- STATS calls:      none
c ---------------------------------------------------------------------
c
      real xobs(n),ypre(n)
      character*4 wdflag
      logical LWIND

c --- Use double precision for the stats
      real*8 rn8,xobs8,ypre8,zero8
      real*8 x8,y8,xx8,yy8,xy8,ar8,rr8,rx8,ry8,dioa8
      real*8 xomean8,ypmean8,bias8,gross8,rmse8,rmses8,rmseu8,rioa8
      real*8 a8,b8

      real*8 top8,denom8

      data zero8/0.d0/
      data dum/-999.0/

      LWIND=.FALSE.
      if(wdflag.EQ.'WDIR' .OR. wdflag.EQ.'wdir') LWIND=.TRUE.

      if(n.LT.1) stop 'STATS called with NO DATA'
      rn8=DFLOAT(n)
      rni8=1.d0/rn8

c --- Initialize sums of predicted, observed and products
      x8=zero8
      y8=zero8
      xx8=zero8
      yy8=zero8
      xy8=zero8
      ar8=zero8
      rr8=zero8
      rx8=zero8
      ry8=zero8
      dioa8=zero8

c --- Loop over the data in the arrays and store sums
      do k=1,n
         xobs8=DBLE(xobs(k))
         ypre8=DBLE(ypre(k))

c ---    Wind direction difference bounded by +/-180 degrees
c ---    and used as predicted with observed set to zero
         if(LWIND) then
            pminuso=ypre(k)-xobs(k)
            if(pminuso.GT.180.0) then
               pminuso=pminuso-360.
            elseif(pminuso.LT.-180.0) then
               pminuso=pminuso+360.
            endif
            xobs8=0.d0
            ypre8=DBLE(pminuso)
         endif

         x8=x8+xobs8
         y8=y8+ypre8
         xx8=xx8+xobs8*xobs8
         xy8=xy8+xobs8*ypre8
         yy8=yy8+ypre8*ypre8
         ar8=ar8+DABS(ypre8-xobs8)
      enddo

c --- Compute immediate means
      xomean8=x8*rni8
      ypmean8=y8*rni8
      bias8=ypmean8-xomean8
      gross8=ar8*rni8
      rmse8=DSQRT(rni8*(yy8+xx8-2.d0*xy8))

c --- Linear least-squares fit of prediction set
c --- to observed set (regressed predictions)
c --- (rpre = a + b*xobs)
c --- Compute a and b

      top8=(rn8*xy8-x8*y8)
      denom8=(rn8*xx8-x8*x8)
      if(denom8.EQ.zero8) then
         b8=zero8
         if(top8.NE.zero8) then
            do i=1,n
               write(*,*)'xobs,ypre = ',xobs(i),ypre(i)
            enddo
            write(*,*)'Numerator,Denomenator = ',top8,denom8
         endif
      else
         b8=top8/denom8
      endif
      a8=(y8-b8*x8)*rni8

c --- Compute additional sums
      do k=1,n
         xobs8=DBLE(xobs(k))
         ypre8=DBLE(ypre(k))
         if(LWIND) then
            pminuso=ypre(k)-xobs(k)
            if(pminuso.GT.180.0) then
               pminuso=pminuso-360.
            elseif(pminuso.LT.-180.0) then
               pminuso=pminuso+360.
            endif
            xobs8=0.d0
            ypre8=DBLE(pminuso)
         endif
         dioa8=dioa8+(DABS(ypre8-xomean8)+DABS(xobs8-xomean8))**2
         rpre8=a8+b8*xobs8
         rr8=rr8+rpre8*rpre8
         ry8=ry8+rpre8*ypre8
         rx8=rx8+rpre8*xobs8
      enddo

c --- Compute remaining means
      if(dioa8.EQ.zero8) then
         if(rmse8.EQ.zero8) then
            rioa8=1.d0
         else
            write(*,*)'Denomenator dioa8 = ',dioa8
            write(*,*)'Number of differences = ',n
            stop 'HALTED in STATS'
         endif
      else
         rioa8=1.d0-rn8*rmse8*rmse8/dioa8
      endif
      rmses8=DSQRT(rni8*(rr8+xx8-2.d0*rx8))
      rmseu8=DSQRT(rni8*(rr8+yy8-2.d0*ry8))

c --- Return all means as single-precision
      xomean=SNGL(xomean8)
      ypmean=SNGL(ypmean8)
      bias=SNGL(bias8)
      gross=SNGL(gross8)
      rmse=SNGL(rmse8)
      rmses=SNGL(rmses8)
      rmseu=SNGL(rmseu8)
      rioa=SNGL(rioa8)
      ra=SNGL(a8)
      rb=SNGL(b8)

      if(LWIND) then
c ---    Wind direction output just bias and error
         xomean=dum
         ypmean=dum
         bias=SNGL(bias8)
         gross=SNGL(gross8)
         rmse=dum
         rmses=dum
         rmseu=dum
         rioa=dum
         ra=SNGL(a8)
         rb=SNGL(b8)
      endif

      return
      end


c ***********************************************************************
      subroutine noblank(string,nt)

c     Get file name from an input line 
c     The file name is at the begining of and before the first 
c     blank space in the line

c -------------------------------------------------------------------
      character*80 string

      do i=80,1,-1
         if(string(i:i).ne.' ') then
            nt=i
            return
         endif
      enddo
        
      nt=10
      
      return
      end


c --------------------------------------------------------------
      Subroutine getoffpath(buff,nt2)

c     Get rid off path part of a file name

      include 'metcomp.par'
      character*80 buff

      nt=index(buff,' ')-1
      if(nt.lt.1) then
         write(ilog,*)'Error: Blank string for file name'
         print *,'Error: Blank string for file name'
         stop 31
      endif

      do i=nt,1,-1
c         if(buff(i:i).eq.'/' .or. buff(i:i).eq.'\\') then
         if(buff(i:i).eq.'/' .or. buff(i:i).eq.'\') then
            nt2=i+1
            return
         endif
      enddo

      nt2=1
c      print *,'No Path in File name: ',buff(1:nt)
      return
      end


c----------------------------------------------------------------------
      subroutine store
c ---------------------------------------------------------------------
c
c --- METCOMP    Version: 3.21       Level: 060630                STORE
c ---            D. Strimaitis
c
c --- Pass non-missing array values to storage arrays for summary
c     processing at end of run
c
c --- INPUTS:
c --- OUTPUT:
c
c --- STORE called by:  main
c --- STORE calls:      none
c ---------------------------------------------------------------------
c
      include 'metcomp.par'
      include 'metcomp.cmn'

c --- Unavailable measures are -999. (dum)

c --- Hourly
      do k=1,mxhr1
c ---    BIAS
         if(sbias(k).GT.-900.) then
            nsbias=nsbias+1
            rsbias(nsbias)=sbias(k)
         endif
         if(dbias(k).GT.-900.) then
            ndbias=ndbias+1
            rdbias(ndbias)=dbias(k)
         endif
         if(tbias(k).GT.-900.) then
            ntbias=ntbias+1
            rtbias(ntbias)=tbias(k)
         endif
         if(qbias(k).GT.-900.) then
            nqbias=nqbias+1
            rqbias(nqbias)=qbias(k)
         endif
c ---    GROSS ERROR
         if(sgerr(k).GT.-900.) then
            nsgerr=nsgerr+1
            rsgerr(nsgerr)=sgerr(k)
         endif
         if(dgerr(k).GT.-900.) then
            ndgerr=ndgerr+1
            rdgerr(ndgerr)=dgerr(k)
         endif
         if(tgerr(k).GT.-900.) then
            ntgerr=ntgerr+1
            rtgerr(ntgerr)=tgerr(k)
         endif
         if(qgerr(k).GT.-900.) then
            nqgerr=nqgerr+1
            rqgerr(nqgerr)=qgerr(k)
         endif

c ---    RMSE
         if(srmse(k).GT.-900.) then
            nsrmse=nsrmse+1
            rsrmse(nsrmse)=srmse(k)
            nsrmser=nsrmser+1
            if(srmse(k).GT.0.0) then
               rsrmser(nsrmser)=srmses(k)/srmse(k)
            else
               rsrmser(nsrmser)=1.0
            endif
         endif
         if(trmse(k).GT.-900.) then
            ntrmse=ntrmse+1
            rtrmse(ntrmse)=trmse(k)
            ntrmser=ntrmser+1
            if(trmse(k).GT.0.0) then
               rtrmser(ntrmser)=trmses(k)/trmse(k)
            else
               rtrmser(ntrmser)=1.0
            endif
         endif
         if(qrmse(k).GT.-900.) then
            nqrmse=nqrmse+1
            rqrmse(nqrmse)=qrmse(k)
            nqrmser=nqrmser+1
            if(qrmse(k).GT.0.0) then
               rqrmser(nqrmser)=qrmses(k)/qrmse(k)
            else
               rqrmser(nqrmser)=1.0
            endif
         endif
c ---    IOA
         if(sioa(k).GT.-900.) then
            nsioa=nsioa+1
            rsioa(nsioa)=sioa(k)
         endif
         if(tioa(k).GT.-900.) then
            ntioa=ntioa+1
            rtioa(ntioa)=tioa(k)
         endif
         if(qioa(k).GT.-900.) then
            nqioa=nqioa+1
            rqioa(nqioa)=qioa(k)
         endif
      enddo

c --- Daily
c --- BIAS
      if(sbias24.GT.-900.) then
         nsbias24=nsbias24+1
         rsbias24(nsbias24)=sbias24
      endif
      if(dbias24.GT.-900.) then
         ndbias24=ndbias24+1
         rdbias24(ndbias24)=dbias24
      endif
      if(tbias24.GT.-900.) then
         ntbias24=ntbias24+1
         rtbias24(ntbias24)=tbias24
      endif
      if(qbias24.GT.-900.) then
         nqbias24=nqbias24+1
         rqbias24(nqbias24)=qbias24
      endif
c --- GROSS ERROR
      if(sgerr24.GT.-900.) then
         nsgerr24=nsgerr24+1
         rsgerr24(nsgerr24)=sgerr24
      endif
      if(dgerr24.GT.-900.) then
         ndgerr24=ndgerr24+1
         rdgerr24(ndgerr24)=dgerr24
      endif
      if(tgerr24.GT.-900.) then
         ntgerr24=ntgerr24+1
         rtgerr24(ntgerr24)=tgerr24
      endif
      if(qgerr24.GT.-900.) then
         nqgerr24=nqgerr24+1
         rqgerr24(nqgerr24)=qgerr24
      endif
c --- RMSE
      if(srmse24.GT.-900.) then
         nsrmse24=nsrmse24+1
         rsrmse24(nsrmse24)=srmse24
         nsrmser24=nsrmser24+1
         if(srmse24.GT.0.0) then
            rsrmser24(nsrmser24)=srmses24/srmse24
         else
            rsrmser24(nsrmser24)=1.0
         endif
      endif
      if(trmse24.GT.-900.) then
         ntrmse24=ntrmse24+1
         rtrmse24(ntrmse24)=trmse24
         ntrmser24=ntrmser24+1
         rtrmser24(ntrmser24)=trmses24/trmse24
         if(trmse24.GT.0.0) then
            rtrmser24(ntrmser24)=trmses24/trmse24
         else
            rtrmser24(ntrmser24)=1.0
         endif
      endif
      if(qrmse24.GT.-900.) then
         nqrmse24=nqrmse24+1
         rqrmse24(nqrmse24)=qrmse24
         nqrmser24=nqrmser24+1
         rqrmser24(nqrmser24)=qrmses24/qrmse24
         if(qrmse24.GT.0.0) then
            rqrmser24(nqrmser24)=qrmses24/qrmse24
         else
            rqrmser24(nqrmser24)=1.0
         endif
      endif
c --- IOA
      if(sioa24.GT.-900.) then
         nsioa24=nsioa24+1
         rsioa24(nsioa24)=sioa24
      endif
      if(tioa24.GT.-900.) then
         ntioa24=ntioa24+1
         rtioa24(ntioa24)=tioa24
      endif
      if(qioa24.GT.-900.) then
         nqioa24=nqioa24+1
         rqioa24(nqioa24)=qioa24
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine summary
c ---------------------------------------------------------------------
c
c --- METCOMP    Version: 3.21       Level: 060630              SUMMARY
c ---            D. Strimaitis
c
c --- Process hourly and daily statistics and report to output files
c
c --- INPUTS:
c --- OUTPUT:
c
c --- SUMMARY called by:  main
c --- SUMMARY calls:      DESCRIBE, BENCHMARK
c ---------------------------------------------------------------------
c
      include 'metcomp.par'
      include 'metcomp.cmn'

      non=0

c --- Hourly
c ----------

c --- Get descriptive stats for each statistical measure
c --- Wind Speed
      call DESCRIBE(rsbias,nsbias,sbiasav,sbiasmn,sbiasmx)
      call DESCRIBE(rsgerr,nsgerr,sgerrav,sgerrmn,sgerrmx)
      call DESCRIBE(rsrmse,nsrmse,srmseav,srmsemn,srmsemx)
      call DESCRIBE(rsrmser,nsrmser,srmserav,srmsermn,srmsermx)
      if(srmserav.LT.-900.) then
         srmserav=1.0
         srmsermx=1.0
         srmsermn=1.0
      endif

      call DESCRIBE(rsioa,nsioa,sioaav,sioamn,sioamx)

c --- Wind Direction
      call DESCRIBE(rdbias,ndbias,dbiasav,dbiasmn,dbiasmx)
      call DESCRIBE(rdgerr,ndgerr,dgerrav,dgerrmn,dgerrmx)

c --- Temperature
      call DESCRIBE(rtbias,ntbias,tbiasav,tbiasmn,tbiasmx)
      call DESCRIBE(rtgerr,ntgerr,tgerrav,tgerrmn,tgerrmx)
      call DESCRIBE(rtrmse,ntrmse,trmseav,trmsemn,trmsemx)
      call DESCRIBE(rtrmser,ntrmser,trmserav,trmsermn,trmsermx)
      if(trmserav.LT.-900.) then
         trmserav=1.0
         trmsermx=1.0
         trmsermn=1.0
      endif
      call DESCRIBE(rtioa,ntioa,tioaav,tioamn,tioamx)
c --- Specific Humidity
      call DESCRIBE(rqbias,nqbias,qbiasav,qbiasmn,qbiasmx)
      call DESCRIBE(rqgerr,nqgerr,qgerrav,qgerrmn,qgerrmx)
      call DESCRIBE(rqrmse,nqrmse,qrmseav,qrmsemn,qrmsemx)
      call DESCRIBE(rqrmser,nqrmser,qrmserav,qrmsermn,qrmsermx)
      if(qrmserav.LT.-900.) then
         qrmserav=1.0
         qrmsermx=1.0
         qrmsermn=1.0
      endif
      call DESCRIBE(rqioa,nqioa,qioaav,qioamn,qioamx)

c --- Daily
c ----------

c --- Get descriptive stats for each statistical measure
c --- Wind Speed
      call DESCRIBE(rsbias24,nsbias24,sbiasav24,sbiasmn24,sbiasmx24)
      call DESCRIBE(rsgerr24,nsgerr24,sgerrav24,sgerrmn24,sgerrmx24)
      call DESCRIBE(rsrmse24,nsrmse24,srmseav24,srmsemn24,srmsemx24)
      call DESCRIBE(rsrmser24,nsrmser24,srmserav24,srmsermn24,
     &              srmsermx24)
      if(srmserav24.LT.-900.) then
         srmserav24=1.0
         srmsermx24=1.0
         srmsermn24=1.0
      endif
      call DESCRIBE(rsioa24,nsioa24,sioaav24,sioamn24,sioamx24)
c --- Wind Direction
      call DESCRIBE(rdbias24,ndbias24,dbiasav24,dbiasmn24,dbiasmx24)
      call DESCRIBE(rdgerr24,ndgerr24,dgerrav24,dgerrmn24,dgerrmx24)
c --- Temperature
      call DESCRIBE(rtbias24,ntbias24,tbiasav24,tbiasmn24,tbiasmx24)
      call DESCRIBE(rtgerr24,ntgerr24,tgerrav24,tgerrmn24,tgerrmx24)
      call DESCRIBE(rtrmse24,ntrmse24,trmseav24,trmsemn24,trmsemx24)
      call DESCRIBE(rtrmser24,ntrmser24,trmserav24,trmsermn24,
     &              trmsermx24)
      if(trmserav24.LT.-900.) then
         trmserav24=1.0
         trmsermx24=1.0
         trmsermn24=1.0
      endif
      call DESCRIBE(rtioa24,ntioa24,tioaav24,tioamn24,tioamx24)
c --- Specific Humidity
      call DESCRIBE(rqbias24,nqbias24,qbiasav24,qbiasmn24,qbiasmx24)
      call DESCRIBE(rqgerr24,nqgerr24,qgerrav24,qgerrmn24,qgerrmx24)
      call DESCRIBE(rqrmse24,nqrmse24,qrmseav24,qrmsemn24,qrmsemx24)
      call DESCRIBE(rqrmser24,nqrmser24,qrmserav24,qrmsermn24,
     &              qrmsermx24)
      if(qrmserav24.LT.-900.) then
         qrmserav24=1.0
         qrmsermx24=1.0
         qrmsermn24=1.0
      endif
      call DESCRIBE(rqioa24,nqioa24,qioaav24,qioamn24,qioamx24)

c --- Assess values relative to benchmarks
c --- Wind Speed
      sbiasmark=0.5
      srmsemark=2.0
      sioamark=0.6
      call BENCHMARK(rsbias24,nsbias24,sbiasmark,sbiasless,sbiasmore)
      call BENCHMARK(rsrmse24,nsrmse24,srmsemark,srmseless,srmsemore)
      call BENCHMARK(rsioa24,nsioa24,sioamark,sioaless,sioamore)
c --- Wind Direction
      dbiasmark=10.0
      dgerrmark=30.0
      call BENCHMARK(rdbias24,ndbias24,dbiasmark,dbiasless,dbiasmore)
      call BENCHMARK(rdgerr24,ndgerr24,dgerrmark,dgerrless,dgerrmore)
c --- Temperature
      tbiasmark=0.5
      tgerrmark=2.0
      tioamark=0.8
      call BENCHMARK(rtbias24,ntbias24,tbiasmark,tbiasless,tbiasmore)
      call BENCHMARK(rtgerr24,ntgerr24,tgerrmark,tgerrless,tgerrmore)
      call BENCHMARK(rtioa24,ntioa24,tioamark,tioaless,tioamore)
c --- Specific Humidity
      qbiasmark=1.0
      qgerrmark=2.0
      qioamark=0.6
      call BENCHMARK(rqbias24,nqbias24,qbiasmark,qbiasless,qbiasmore)
      call BENCHMARK(rqgerr24,nqgerr24,qgerrmark,qgerrless,qgerrmore)
      call BENCHMARK(rqioa24,nqioa24,qioamark,qioaless,qioamore)


c --- Tabulate in output files
c ----------------------------

c --- Maximum
      write(io1,1126) 'Maximum      ',
     &           sioamx,sbiasmx,sgerrmx24,srmsemx,srmsermx,
     &          (1.-srmsermx),dbiasmx,dgerrmx
      write(io2,1126) 'Maximum      ',
     &           sioamx24,sbiasmx24,sgerrmx24,srmsemx24,srmsermx24,
     &           (1.-srmsermx24),dbiasmx24,dgerrmx24
      write(io3,1127) 'Maximum      ',
     &           tioamx,tbiasmx,tgerrmx,trmsemx,trmsermx,
     &           (1.-trmsermx),qioamx,qbiasmx,qgerrmx,qrmsemx,
     &           qrmsermx,(1.-qrmsermx)
      write(io4,1127) 'Maximum      ',
     &           tioamx24,tbiasmx24,tgerrmx24,trmsemx24,trmsermx24,
     &           (1.-trmsermx24),
     &           qioamx24,qbiasmx24,qgerrmx24,qrmsemx24,qrmsermx24,
     &           (1.-qrmsermx24)
c --- Minimum
      write(io1,1126) 'Minimum      ',
     &           sioamn,sbiasmn,sgerrmn,srmsemn,srmsermn,(1.-srmsermn),
     &           dbiasmn,dgerrmn
      write(io2,1126) 'Minimum      ',
     &           sioamn24,sbiasmn24,sgerrmn24,srmsemn24,srmsermn24,
     &           (1.-srmsermn24),
     &           dbiasmn24,dgerrmn24
      write(io3,1127) 'Minimum      ',
     &           tioamn,tbiasmn,tgerrmn,trmsemn,trmsermn,(1.-trmsermn),
     &           qioamn,qbiasmn,qgerrmn,qrmsemn,qrmsermn,(1.-qrmsermn)
      write(io4,1127) 'Minimum      ',
     &           tioamn24,tbiasmn24,tgerrmn24,trmsemn24,trmsermn24,
     &           (1.-trmsermn24),
     &           qioamn24,qbiasmn24,qgerrmn24,qrmsemn24,qrmsermn24,
     &           (1.-qrmsermn24)
c --- Mean
      write(io1,1126) 'Mean         ',
     &           sioaav,sbiasav,sgerrav,srmseav,srmserav,(1.-srmserav),
     &           dbiasav,dgerrav
      write(io2,1126) 'Mean         ',
     &           sioaav24,sbiasav24,sgerrav24,srmseav24,srmserav24,
     &           (1.-srmserav24),
     &           dbiasav24,dgerrav24
      write(io3,1127) 'Mean         ',
     &           tioaav,tbiasav,tgerrav,trmseav,trmserav,(1.-trmserav),
     &           qioaav,qbiasav,qgerrav,qrmseav,qrmserav,(1.-qrmserav)
      write(io4,1127) 'Mean         ',
     &           tioaav24,tbiasav24,tgerrav24,trmseav24,trmserav24,
     &           (1.-trmserav24),
     &           qioaav24,qbiasav24,qgerrav24,qrmseav24,qrmserav24,
     &           (1.-qrmserav24)

c --- Benchmarks
      write(io2,*)
      write(io2,1226) 'Benchmark    ',
     &           sioamark,sbiasmark,srmsemark,
     &           dbiasmark,dgerrmark
      write(io2,1226) 'Fraction LESS',
     &           sioaless,sbiasless,srmseless,
     &           dbiasless,dgerrless
      write(io2,1226) 'Fraction MORE',
     &           sioamore,sbiasmore,srmsemore,
     &           dbiasmore,dgerrmore
      write(io4,*)
      write(io4,1227) 'Benchmark    ',
     &           tioamark,tbiasmark,tgerrmark,
     &           qioamark,qbiasmark,qgerrmark
      write(io4,1227) 'Fraction LESS',
     &           tioaless,tbiasless,tgerrless,
     &           qioaless,qbiasless,qgerrless
      write(io4,1227) 'Fraction MORE',
     &           tioamore,tbiasmore,tgerrmore,
     &           qioamore,qbiasmore,qgerrmore

c --- Number
      write(io1,*)
      write(io1,1026) 'Total Number ',
     &      nsioa,nsbias,nsgerr,nsrmse,nsrmser,nsrmser,
     &      ndbias,ndgerr
      write(io2,*)
      write(io2,1026) 'Total Number ',
     &      nsioa24,nsbias24,nsgerr24,nsrmse24,nsrmser24,nsrmser24,
     &      ndbias24,ndgerr24
      write(io3,*)
      write(io3,1027) 'Total Number ',
     &      ntioa,ntbias,ntgerr,ntrmse,ntrmser,ntrmser,
     &      nqioa,nqbias,nqgerr,nqrmse,nqrmser,nqrmser
      write(io4,*)
      write(io4,1027) 'Total Number ',
     &      ntioa24,ntbias24,ntgerr24,ntrmse24,ntrmser24,ntrmser24,
     &      nqioa24,nqbias24,nqgerr24,nqrmse24,nqrmser24,nqrmser24
     &           

 1026 format(a13,i8,14x,2i9,3i8,16x,2i8)
 1027 format(a13,2(i8,14x,2i9,3i8))
 1126 format(a13,f8.2,14x,2f9.2,3f8.2,16x,2f8.2)
 1127 format(a13,2(f8.2,14x,2f9.2,3f8.2))
 1226 format(a13,f8.2,14x,f9.2,9x,f8.2,8x,8x,16x,2f8.2)
 1227 format(a13,2(f8.2,14x,2f9.2,24x))



c --- Tabulate in list file
c -------------------------


      write(ilog,'(/)')
      write(ilog,'(a)')
     &   '---------------------------------------------------------',
     &   'Wind Speed (m/s)',
     &   '---------------------------------------------------------'
      write(ilog,'(2a30)')'                     IOA    Bi',
     &                    'as    RMSE   RMSES   Gross    '
      write(ilog,'(2a30)')'                              ',
     &                    '             /RMSE   Error    '
      write(ilog,'(a16,5f8.2)') 'Maximum         ',
     &               sioamx24,sbiasmx24,srmsemx24,srmsermx24,sgerrmx24
      write(ilog,'(a16,5f8.2)') 'Minimum         ',
     &               sioamn24,sbiasmn24,srmsemn24,srmsermn24,sgerrmn24
      write(ilog,'(a16,5f8.2)') 'Mean            ',
     &               sioaav24,sbiasav24,srmseav24,srmserav24,sgerrav24
      write(ilog,'(a16,3f8.2)')    'BENCHMARK       ',
     &                              sioamark,sbiasmark,srmsemark
      write(ilog,'(a16,3f8.2)') 'Fraction meeting',sioamore,sbiasless,
     &                                             srmseless
      write(ilog,'(a16)')       '   the benchmark'
     &               
      write(ilog,*)
      write(ilog,'(a16,5i8)')   'Total # Days    ',
     &               nsioa24,nsbias24,nsrmse24,nsrmser24,nsgerr24

      write(ilog,'(/)')
      write(ilog,'(a)')
     &   '---------------------------------------------------------',
     &   'Wind Direction (deg)',
     &   '---------------------------------------------------------'
      write(ilog,'(a)')'                    Bias   Gross'
      write(ilog,'(a)')'                           Error'
      write(ilog,'(a16,2f8.2)') 'Maximum         ',dbiasmx24,dgerrmx24
      write(ilog,'(a16,2f8.2)') 'Minimum         ',dbiasmn24,dgerrmn24
      write(ilog,'(a16,2f8.2)') 'Mean            ',dbiasav24,dgerrav24
      write(ilog,'(a16,2f8.2)') 'BENCHMARK       ',dbiasmark,dgerrmark
      write(ilog,'(a16,2f8.2)') 'Fraction meeting',dbiasless,dgerrless
      write(ilog,'(a16)')       '   the benchmark'
      write(ilog,*)
      write(ilog,'(a16,2i8)')   'Total # Days    ',ndbias24,ndgerr24

      write(ilog,'(/)')
      write(ilog,'(a)')
     &   '---------------------------------------------------------',
     &   'Temperature (K)',
     &   '---------------------------------------------------------'
      write(ilog,'(2a30)')'                     IOA    Bi',
     &                    'as   Gross    RMSE   RMSES    '
      write(ilog,'(2a30)')'                              ',
     &                    '     Error           /RMSE    '
      write(ilog,'(a16,5f8.2)') 'Maximum         ',
     &               tioamx24,tbiasmx24,tgerrmx24,trmsemx24,trmsermx24
      write(ilog,'(a16,5f8.2)') 'Minimum         ',
     &               tioamn24,tbiasmn24,tgerrmn24,trmsemn24,trmsermn24
      write(ilog,'(a16,5f8.2)') 'Mean            ',
     &               tioaav24,tbiasav24,tgerrav24,trmseav24,trmserav24
      write(ilog,'(a16,3f8.2)')    'BENCHMARK       ',
     &                              tioamark,tbiasmark,tgerrmark
      write(ilog,'(a16,3f8.2)') 'Fraction meeting',tioamore,tbiasless,
     &                                             tgerrless
      write(ilog,'(a16)')       '   the benchmark'
      write(ilog,*)
      write(ilog,'(a16,5i8)')   'Total # Days    ',
     &               ntioa24,ntbias24,ntgerr24,ntrmse24,ntrmser24

      write(ilog,'(/)')
      write(ilog,'(a)')
     &   '---------------------------------------------------------',
     &   'Specific Humidity (g/kg)',
     &   '---------------------------------------------------------'
      write(ilog,'(2a30)')'                     IOA    Bi',
     &                    'as   Gross    RMSE   RMSES    '
      write(ilog,'(2a30)')'                              ',
     &                    '     Error           /RMSE    '
      write(ilog,'(a16,5f8.2)') 'Maximum         ',
     &               qioamx24,qbiasmx24,qgerrmx24,qrmsemx24,qrmsermx24
      write(ilog,'(a16,5f8.2)') 'Minimum         ',
     &               qioamn24,qbiasmn24,qgerrmn24,qrmsemn24,qrmsermn24
      write(ilog,'(a16,5f8.2)') 'Mean            ',
     &               qioaav24,qbiasav24,qgerrav24,qrmseav24,qrmserav24
      write(ilog,'(a16,3f8.2)') 'BENCHMARK       ',
     &                           qioamark,qbiasmark,qgerrmark
      write(ilog,'(a16,3f8.2)') 'Fraction meeting',qioamore,qbiasless,
     &                                             qgerrless
      write(ilog,'(a16)')       '   the benchmark'
      write(ilog,*)
      write(ilog,'(a16,5i8)')  'Total # Days    ',
     &               nqioa24,nqbias24,nqgerr24,nqrmse24,nqrmser24


      return
      end


c----------------------------------------------------------------------
      subroutine describe(val,n,vmean,vmin,vmax)
c ---------------------------------------------------------------------
c
c --- METCOMP    Version: 3.21       Level: 060630             DESCRIBE
c ---            D. Strimaitis
c
c --- Compute descriptive measures MEAN, MAXIMUM, MINIMUM for values
c     in an array
c
c --- INPUTS:
c              VAL - real array - Set of real numbers
c                N - integer    - Number of elements in VAL
c --- OUTPUT:
c            VMEAN - real       - Mean of VAL
c             VMIN - real       - Minimum value in VAL
c             VMAX - real       - Maximum value in VAL
c
c --- DESCRIBE called by:  main
c --- DESCRIBE calls:      none
c ---------------------------------------------------------------------
c
      real val(n)
      dum=-999.0

      vmean=dum
      vmin=dum
      vmax=dum
      if(n.LT.1) return

c --- Start with first value
      vmean=val(1)
      vmin=val(1)
      vmax=val(1)

c --- Loop over remaining values
      do k=2,n
         vmean=vmean+val(k)
         if(vmin.GT.val(k)) vmin=val(k)
         if(vmax.LT.val(k)) vmax=val(k)
      enddo

c --- Turn sum into average
      vmean=vmean/FLOAT(n)

      return
      end

c----------------------------------------------------------------------
      subroutine benchmark(val,n,vmark,fless,fmore)
c ---------------------------------------------------------------------
c
c --- METCOMP    Version: 3.21       Level: 060630            BENCHMARK
c ---            D. Strimaitis
c
c --- Count fraction of ABS(data) in array that is
c         .LE. benchmark value or
c         .GE. benchmark value
c     and report both (need not sum to 1.0)
c
c --- INPUTS:
c              VAL - real array - Set of real numbers
c                N - integer    - Number of elements in VAL
c            VMARK - real       - Benchmark value
c --- OUTPUT:
c            FLESS - real       - Fraction .LE. benchmark
c            FMORE - real       - Fraction .GE. benchmark
c
c --- BENCHMARK called by:  main
c --- BENCHMARK calls:      none
c ---------------------------------------------------------------------
c
      real val(n)

      fless=0.0
      fmore=0.0
      iless=0
      imore=0
      if(n.LT.1) return

c --- Loop over values
      do k=1,n
         aval=ABS(val(k))
         if(aval.LE.vmark) iless=iless+1
         if(aval.GE.vmark) imore=imore+1
      enddo

c --- Turn counts into fraction
      fless=FLOAT(iless)/FLOAT(n)
      fmore=FLOAT(imore)/FLOAT(n)

      return
      end

c ***********************************************************************
      subroutine wdfreq (wss,wds,freq,wsfm,npts,nodata)

c Purpose: Calculate windrose frequency at a single position 

      include 'metcomp.par'
      include 'metcomp.cmn'

      logical ldata

      dimension wss(mxhours),wds(mxhours)
      dimension freq(nws1,ndir1),wsfm(nws1,ndir1),ipp(nws1,ndir1)

c wss:     Wind speed
c wds:     Wind direction
c npts:    Number of points of wss and wds series
c ntotal:  Number of Non-missing points of wss and wds time series
c ncalm:   Number of calm wind points
c nws:     Wind speed class
c ndir:    Number of wind scales
c wscalm:  Calm wind speed
c freq:    Frequency array
c nodata:  0=data exists, 1=all missing
c ------------------------------------------------------------------------

c     Initial counter array
      do j=1,ndir1
         do i=1,nws1
            freq(i,j)=0
            wsfm(i,j)=0
            ipp(i,j)=0
         enddo
      enddo

      if(npts.lt.1) then
c         write(ilog,*)'WDFREQ: No Data in WDS/WSS time series'
c         write(ilog,*)'Frequencies are not calculated'

c         print *,'WDFREQ: No data in WDS/WSS time series'
c         print *,'Frequencies are not calculated'

         nodata=1

         return

      endif

      nmiss=0
      ncalm=0

      do ih=1,npts
         ss=wss(ih)
         dd=wds(ih)
         if(ss.ne.fmiss .and. dd.ne.fmiss) then
            ihbeg=ih
            goto 500
         endif
         nmiss=nmiss+1
      enddo

c      write(ilog,*)'All data in WDS/WSS time series are missing'
c      write(ilog,*)'Frequencies are not calculated'
c      write(ilog,*)'Exit from wdfreq'

c      print *,'All data in WDS/WSS time series are missing'
c      print *,'Frequencies are not calculated'
c      print *,'Exit from wdfreq'

      nodata=1

      return
	   
 500  continue

      do 1000 ih=ihbeg,npts     ! wind frequency class
         ss=wss(ih)
         dd=wds(ih)

         if(ss.eq.fmiss .or. dd.eq.fmiss) then
            nmiss=nmiss+1
            goto 1000
         endif

c	     Changed on 4/13/2001
c         if(ss.lt.wscalm .and. dd.eq.0) then
         if(ss.lt.wscalm) then
            ncalm=ncalm+1
            goto 1000
         endif
	      
         idc=nint(dd/22.5)+1
         if(idc.gt.ndir) idc=1  

         do i=1,nws
            ss2=wscls(i)
            if(i.eq.1) then
               ss1=0
            else
               ss1=wscls(i-1)
            endif
		
            if(ss.ge.ss1 .and. ss.lt.ss2) then
               isc=i
               goto 1050
            endif
         enddo

         write(ilog,*)'Invalid wind speed: ',ss
         print *,'Invalid wind speed: ',ss
         stop 12

 1050    continue

c        Sum of wind speed class
         ipp(isc,idc)=ipp(isc,idc)+1
         wsfm(isc,idc)=wsfm(isc,idc)+ss

c        Sum of all wind speed class
         ipp(nws1,idc)=ipp(nws1,idc)+1
         wsfm(nws1,idc)=wsfm(nws1,idc)+ss

         ipp(nws1,ndir1)=ipp(nws1,ndir1)+1
         wsfm(nws1,ndir1)=wsfm(nws1,ndir1)+ss

c        Sum of all wind directions
         ipp(isc,ndir1)=ipp(isc,ndir1)+1
         wsfm(isc,ndir1)=wsfm(isc,ndir1)+ss

 1000 enddo

c     QC for sums of points and frequencies
      ntot=0
      do j=1,ndir
         do i=1,nws
            nn=ipp(i,j)
            ntot=ntot+nn
         enddo
      enddo

      ntotal=ntot+ncalm

      nall=ntotal+nmiss
      if(nall.ne.npts) then
         write(ilog,*)'Error: Total Number not Match:',nall,npts
         stop 10
      endif

      atotal=float(ntotal)

      fall=0
      do j=1,ndir1
         do i=1,nws1
            freq(i,j)=ipp(i,j)/atotal
            if(j.ne.ndir1 .and. i.ne.nws1) fall=fall+freq(i,j)
         enddo
      enddo
	      		
      frecalm=float(ncalm)/atotal*100.

      fall=fall*100+frecalm

      dfreq=abs(100-fall)
      
      if(dfreq .gt. 1.0) then
         write(ilog,*)'Error in summed frequency:'
         write(ilog,*)fall
         print *,'Error in summed frequency:'
         print *,fall
         stop 11
      endif

c     Get frequency mean wind speed
      do j=1,ndir1
         do i=1,nws1
            ip=ipp(i,j)
            aa=wsfm(i,j)
            if(ip.ge.1) then
               aa=aa/ip
            else
               aa=0
            endif
            wsfm(i,j)=aa
         enddo
      enddo

      return
      end

c-------------------------------------------------------------
      subroutine timestamp(iyr,imon,iday,ihour,idate) 
c-------------------------------------------------------------
c
c --- METCOMP  Version: 3.21       Level: 031126            timestamp
c ---          Zhong Wu
c
c --- purpose:  Get timestamp
c
c --- inputs:
c           iyr   - integer    - year
c           imon  - integer    - month
c           iday  - integer    - day
c           ihour - integer    - hour
c
c --- output:
c           idate - integer      - timestamp              
c
c --- timestamp called by:  main
c --- timestamp calls:      none
c----------------------------------------------------------------------

      iyr2k=iyr
      if(iyr.lt.100) then
         if(iyr.lt.50) then
            iyr2k=iyr+2000
         else
            iyr2k=iyr+1900
         endif
      endif
      idate=ihour+iday*100+imon*10000+iyr2k*1000000

      return
      end
c----------------------------------------------------------------------
      subroutine stamptime(ndate,iyr,imon,iday,ihour)
c----------------------------------------------------------------------
c
c --- METCOMP  Version: 3.21       Level: 031126                Stamptime
c ---          Zhong Wu
c
c --- purpose:  compute gregorian date
c
c --- inputs:
c           iyr - integer      - year
c           imo - integer      - month
c          iday - integer      - day
c          ihour - integer     - hour
c
c --- output:
c          ndate - integer      - date
c
c --- stamptime called by:  main
c --- stamptime calls:      none
c----------------------------------------------------------------------

      include 'metcomp.par'

      iyr=ndate/isix
      imon=ndate/ifour-iyr*100
      iday=ndate/100-iyr*ifour-imon*100
      ihour=ndate-iyr*isix-imon*ifour-iday*100
      
       return
       end

! **********************************************************************
      subroutine chgtim(iyr,imon,iday,ihour,idt)

c --- METCOMP    Version: 1.5       Level: 061103               
! --- Zhong-Xiang Wu           
!
! --- PURPOSE:
!     Increase or decrease hours. 
!     Used to convert Local Stand Time to GMT or to setup time stamp for loop

      parameter(nmonth=12)

      dimension ndays(nmonth)

      data ndays/31,28,31,30,31,30,31,31,30,31,30,31/

      if(mod(iyr,4).eq.0) then
         ndays(2)=29 
      else
         ndays(2)=28
      endif

      ihour=ihour+idt

      if(idt.lt.0) goto 2000

 1000 if(ihour.gt.23) then
         ihour=ihour-24
         iday=iday+1
         
         if(iday.gt.ndays(imon)) then
            iday=1
            imon=imon+1
            
            if(imon.gt.12) then
               imon=1
               iyr=iyr+1
               if(iyr/4 .eq. iyr/4.0) then
                  ndays(2)=29 
               else
                  ndays(2)=28
               endif
            endif
         endif
         goto 1000
      else
         return
      endif

 2000 continue

 3000 if(ihour.lt.0) then
         ihour=ihour+24
         iday=iday-1
         
         if(iday.le.0) then
            imon=imon-1
            if(imon.le.0) then
               iyr=iyr-1
               imon=12
               if(iyr/4 .eq. iyr/4.0) then
                  ndays(2)=29 
               else
                  ndays(2)=28
               endif
            endif
   
            iday=ndays(imon)
         endif
         goto 3000
      else
         return
      endif

      end


c *******************************************************************

      Subroutine freqout(freq,wsfm,npts,idm)

c Purpose: Output wind freqency array in the form of EPA windrose format
c Note:     npts,idgrd and htgrd are exta info and not used by EPA windrose
c           program
c
c --- UPDATES (METSERIES):
c
c Version 1.4, Level 070315 to Version 1.41, Level 070822   (DGS)
c   - Align columns with header in TAB file
c   - Add wind speed standard deviation to "average" table
c
c --- UPDATES:
c
c Version 1.3, Level 060503 to Version 2.0, Level 060605   (DGS)
c   - Change "hour" labels to "period" labels

      include 'metcomp.par'
      include 'metcomp.cmn'

      dimension freq(nws1,ndir1)
      dimension wsfm(nws1,ndir1)

      fcalm=float(ncalm)
      atotal=float(ntotal)
      iht=nint(hgt)

c Output frequency classes in the form of CALWindRose Program 
c      idm=idgrd-idgrd/1000*1000

      nt=index(sname(idm),' ')-1
      write(ifq,*)
      write(ifq,'(a)')sname(idm)(1:nt)

      wscls2(1)=0
      do i=1,nws-1
         wscls2(i+1)=wscls(i)
      enddo

      write(ifq,1142)(wscls2(i),wscls(i),i=1,nws-1),wscls(nws-1)
 1142 format('WDDIR  ',5(f4.1,'-',f4.1,4x),'   > ',f4.1,'          SUM')

      do j=1,ndir1
         write(ifq,1150)sdir(j),(freq(i,j)*100,i=1,nws1)
      enddo
 1150 format(1x,a3,':',7(3x,f7.3,'%',2x))
      write(ifq,1031)frecalm
 1031 format('Calm Winds: ',f7.3,'%')
      write(ifq,1040)npts,nint(atotal),nint(fcalm)
 1040 format('Total Periods = ',i5,';  Valid Periods = ',i5,
     &  ';  Calm Wind Periods = ',i5)

c --- Output frequency classes in tables
      nt=index(sname(idm),' ')-1
      write(itb,1039)sname(idm)(1:nt)
 1039 format(10x,'Wind Statistics: ',a)

      write(itb,1041)npts,nint(atotal),nint(fcalm),iht
 1041 format('Total Points:    ',i5,10x,
     &  'Valid Points:    ',i5,/,
     &  'Calm Wind Points:',i5,10x,
     &  'Elevation(m):    ',i5,/)

      wscls2(1)=0
      do i=1,nws-1
         wscls2(i+1)=wscls(i)
      enddo

      write(itb,1043)
 1043 format(30x,'Frequency Class (%)')
      write(itb,1042)(wscls2(i),wscls(i),i=1,nws-1),wscls(nws-1)
 1042 format('WDDIR',5(2x,f4.1,'-',f4.1),2x,'  > ',f4.1,'      SUM')

      sdir(17)='SUM'
      do j=1,ndir1
         write(itb,1050)sdir(j),(freq(i,j)*100,i=1,nws1)
      enddo
      write(itb,*)
 1050 format(1x,a3,1x,7(2x,f5.1,3x))

c Output wind speed classes in tables
      write(itb,1051)
 1051 format(30x,'Wind Speed Class (m/s)')

      write(itb,1052)(wscls2(i),wscls(i),i=1,nws-1),wscls(nws-1)
 1052 format('WDDIR',5(2x,f4.1,'-',f4.1),2x,'  > ',f4.1,'      AVE')

      sdir(17)='AVE'
      do j=1,ndir1
         write(itb,1050)sdir(j),(wsfm(i,j),i=1,nws1)
      enddo
      write(itb,*)

      return
      end


c----------------------------------------------------------------------
      subroutine ini17zero(i1,i2,i3,i4,i5,i6,i7,i8,i9,
     &           i10,i11,i12,i13,i14,i15,i16,i17)
c ---------------------------------------------------------------------
c
c --- METCOMP    Version: 3.21       Level: 080110            INI17ZERO
c ---           
c
c --- Initial integer zeros
c
c Zhong Wu
c 1/10/2008
c
      
      i1=0
      i2=0
      i3=0
      i4=0
      i5=0
      i6=0
      i7=0
      i8=0
      i9=0
      i10=0
      i11=0
      i12=0
      i13=0
      i14=0
      i15=0
      i16=0
      i17=0

      return
      end

c----------------------------------------------------------------------
      subroutine tclock(io,ndhr,nyr,njul,nhr,nsec)
c----------------------------------------------------------------------
c
c --- CALPUFF    Version: 6.263     Level: 050915                TCLOCK
c                D. Strimaitis,  Earth Tech
c
c --- PURPOSE:  Tests year/day/hour and seconds variables and changes
c               them if needed to fall within standard clock times
c               (HH:SSSS) between 00:0000 and 23:3599
c
c --- INPUTS:
c            IO - integer - List file unit number
c          NDHR - integer - YYYYJJJHH (year-Julian day-hour)
c           NYR - integer - YYYY (year)
c          NJUL - integer - JJJ (Julian day)
c           NHR - integer - HH (hour)
c          NSEC - integer - SSSS (seconds)
c
c
c --- OUTPUT:
c          NDHR - integer - YYYYJJJHH of target (year-day-hour)
c           NYR - integer - YYYY (year)
c          NJUL - integer - JJJ (Julian day)
c           NHR - integer - HH (hour)
c          NSEC - integer - SSSS of target (seconds)
c
c --- TCLOCK called by:  (utility)
c --- TCLOCK calls:      INCR
c----------------------------------------------------------------------

      if(ndhr.GT.0) then
c ---    Extract date/hour elements
         iyr=ndhr/100000
         jjjhh=ndhr-iyr*100000
         jul=jjjhh/100
         ihr=jjjhh-jul*100
      else
c ---    Assign input values
         iyr=nyr
         jul=njul
         ihr=nhr
      endif

      if(nsec.LT.3600 .AND. ihr.LT.24) return

c --- Condition times (HH:SSSS) to be between 00:0000 and 23:3599
      khr=nsec/3600
      nsec=nsec-khr*3600
      ihr=ihr+khr
      if(ihr.GT.23) then
         khr=ihr-23
         ihr=23
         call INCR(io,iyr,jul,ihr,khr)
      endif
      if(ndhr.GT.0) then
         ndhr=iyr*100000+jul*100+ihr
      else
         nyr=iyr
         njul=jul
         nhr=ihr
      endif

      return
      end
